@@ -27,9 +27,12 @@
        192 - 110xxxxx - for U+0080 to U+07FF 
        224 - 1110xxxx - for U+0800 to U+FFFF 
        240 - 11110xxx - for U+010000 to U+1FFFFF */
-static unsigned short mask[] = {192, 224, 240};
+extern unsigned short mask[];// = {192, 224, 240};
 
+#ifndef __BOOL_DEFINED_
+#define __BOOL_DEFINED_
 typedef int bool;
+#endif
 
 char *ugetKey();
 char *ugetKeyUnderConsole();
@@ -37,4 +40,5 @@ char *ugetKeyUnderX();
 
 //the Unicode character returned by ugetKey()
 char uc[5];
-#endif
+
+#endif /* __U_KBD_H */
