@@ -1,22 +1,22 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2014 (c)
+ *    Copyright 2014, 2015 (c)
  * 
  *    file: init.c
- *    This file is part of mino (Mino).
+ *    This file is part of mino.
  *
- *    mino (Mino) is free software: you can redistribute it and/or modify
+ *    mino is free software: you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
  *    the Free Software Foundation, either version 3 of the License, or
  *    (at your option) any later version.
  *
- *    mino (Mino) is distributed in the hope that it will be useful,
+ *    mino is distributed in the hope that it will be useful,
  *    but WITHOUT ANY WARRANTY; without even the implied warranty of
  *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *    GNU General Public License for more details.
  *
  *    You should have received a copy of the GNU General Public License
- *    along with mino (Mino).  If not, see <http://www.gnu.org/licenses/>.
+ *    along with mino.  If not, see <http://www.gnu.org/licenses/>.
  */    
 #include "defs.h"
 #include "kbd.h"
@@ -33,6 +33,7 @@
 int NEW_GNU_DOS_LEVEL;
 struct passwd *pass;//will be used to find the home dir
 char *STARTUP_FILE_NAME;
+char *mino_ver = "1.3";
 
 void exit_gracefully(int exit_code) 
 {
@@ -80,6 +81,8 @@ good_to_go:
   clearScreen(); 
   restoreTerminal(); 
   setScreenColors(WHITE, BGBLACK);
+  fprintf(stdout, "\x1b[24m");
+  fflush(stdout);
   exit(exit_code); 
 }
 
@@ -97,10 +100,10 @@ void parseLineArgs(int argc, char **argv)
 	//parse command line arguments
 	///////////////////////////////////////
 	int c;
-	char *file_name = NULL;
-	static int EXPORT_FLAG = 0;
-	static int LOG_FILE_SET = 0;
-	char *EXPORT_DIR, *EXPORT_FILE;
+	//char *file_name = NULL;
+	//static int EXPORT_FLAG = 0;
+	//static int LOG_FILE_SET = 0;
+	//char *EXPORT_DIR, *EXPORT_FILE;
 	static struct option long_options[] =
 	{
 	 {"reset-config", no_argument,            0,  'r'},
@@ -129,6 +132,7 @@ void parseLineArgs(int argc, char **argv)
 		    exit(1);
 		  }
 		  config_file_name = (char *) malloc(strlen(pass->pw_dir)+12);
+		  if(!config_file_name) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
 		  strcpy(config_file_name, pass->pw_dir);
 		  strcat(config_file_name, "/");
 		  strcat(config_file_name, ".mino.conf");
@@ -195,7 +199,7 @@ void parseLineArgs(int argc, char **argv)
 		  break;
 		case 'h':	//show program help
 		  printf("\nCommand line help for mino under GNU/Linux\n");
-		  printf("Copyright (C) Mohammed Isam 2014\n");
+		  printf("Copyright (C) Mohammed Isam 2014, 2015\n");
 		  printf("This is a GNU software, part of the GnuDOS package\n\n");
 		  printf("Usage: %s [filename][options]\n\n", argv[0]);
 		  printf("Options:\n");
@@ -211,7 +215,7 @@ void parseLineArgs(int argc, char **argv)
 		  exit(0);
 		  break;
 		case 'v':	//show program version
-		  printf("Mino version 1.2\n");
+		  printf("Mino version %s\n", mino_ver);
 		  exit(0);
 		  break;
 		case 'l':	//set GNU_DOS level
@@ -240,122 +244,10 @@ void parseLineArgs(int argc, char **argv)
 	  NEW_FILE = 0; FILE_STATE = OPENED;
 	  open_file_at_startup = 1;
 	  STARTUP_FILE_NAME = (char *)malloc(strlen(argv[optind]));
+	  if(!STARTUP_FILE_NAME) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
 	  strcpy(STARTUP_FILE_NAME, argv[optind]);
 	  break;
 	}
-  
-  /*
-  if(argc > 1) 
-  {//if #1
-   int i;
-   for(i = 1; i < argc; i++)
-    if(strcmp(argv[i], "--level1") == 0 || strcmp(argv[i], "-l1") == 0)
-      NEW_GNU_DOS_LEVEL = 1;
-    else if(strcmp(argv[i], "--level2") == 0 || strcmp(argv[i], "-l2") == 0)
-      NEW_GNU_DOS_LEVEL = 2;
-    else if(strcmp(argv[i], "--level3") == 0 || strcmp(argv[i], "-l3") == 0)
-      NEW_GNU_DOS_LEVEL = 3;
-    else if(strcmp(argv[i], "--level4") == 0 || strcmp(argv[i], "-l4") == 0)
-      NEW_GNU_DOS_LEVEL = 4;
-    else if(strcmp(argv[i], "--level5") == 0 || strcmp(argv[i], "-l5") == 0)
-      NEW_GNU_DOS_LEVEL = 5;
-    else if(strcmp(argv[i], "--reset-config") == 0)
-    {//if #2
-      if(!(pass = getpwuid(geteuid()))) 
-      {
-	printf("Error: couldn't open home directory to write configuration file.\n");
-	printf("Aborting.\n");
-	exit(1);
-      }
-      config_file_name = (char *) malloc(strlen(pass->pw_dir)+11);
-      strcpy(config_file_name, pass->pw_dir);
-      strcat(config_file_name, "/");
-      strcat(config_file_name, ".mino.conf");
-      if(!(config_file = fopen(config_file_name, "w"))) 
-      {
-	printf("Error: couldn't write to configuration file in home directory.\n");
-	printf("Aborting.\n");
-	exit(1);
-      }
-      printf("Resetting program configuration..\n");
-      //write default values to the configuration file
-      fprintf(config_file, "#Configuration file for mino program\n");
-      fprintf(config_file, "#Please do not modify this file by hand\n");
-      fprintf(config_file, "#Use the Options menu in mino to change program configuration.\n\n");
-      //fprintf(config_file, "#Maximum number of dirs that can be handled in dialog boxes\n");
-      //fprintf(config_file, "MAXDIRS = 255\n");
-      //fprintf(config_file, "#Maximum number of files that can be handled in dialog boxes\n");
-      //fprintf(config_file, "MAXFILES = 255\n");
-      //fprintf(config_file, "#Maximum number of permitted lines in each document\n");
-      //fprintf(config_file, "MAXLINES = 100\n\n");
-      fprintf(config_file, "#Line Wrapping\n");
-      fprintf(config_file, "#If TRUE, lines cannot be longer than the screen width\n");
-      fprintf(config_file, "#If FALSE, lines cannot be upto MAX_CHARS_PER_LINE characters\n");
-      fprintf(config_file, "#Note this value is omitted if WRAP_LINES is TRUE\n");
-      fprintf(config_file, "MAX_CHARS_PER_LINE = 100\n");
-      fprintf(config_file, "WRAP_LINES = TRUE\n\n");
-      fprintf(config_file, "#Number of spaces to insert when user presses TAB key\n");
-      fprintf(config_file, "TAB_CHARS = 8\n\n");
-      fprintf(config_file, "#Default title for newly opened documents, no quotes\n");
-      fprintf(config_file, "DEFAULT_TITLE = untitled\n\n");
-      fprintf(config_file, "#Start mino with CAPS and INSERT set to OFF/ON\n");
-      fprintf(config_file, "CAPS = OFF\n");
-      fprintf(config_file, "INSERT = OFF\n\n");
-      fprintf(config_file, "#Display colors\n");
-      fprintf(config_file, "FG_COLOR_WIN = 37\n");
-      fprintf(config_file, "FG_COLOR_HLT = 34\n");
-      fprintf(config_file, "FG_COLOR_MBAR = 34\n");
-      fprintf(config_file, "FG_COLOR_SBAR = 34\n");
-      fprintf(config_file, "FG_COLOR_HBUT = 32\n");
-      fprintf(config_file, "FG_COLOR_BUT = 37\n");
-      fprintf(config_file, "BG_COLOR_WIN = 44\n");
-      fprintf(config_file, "BG_COLOR_HLT = 47\n");
-      fprintf(config_file, "BG_COLOR_MBAR = 47\n");
-      fprintf(config_file, "BG_COLOR_SBAR = 47\n");
-      fprintf(config_file, "BG_COLOR_HBUT = 41\n");
-      fprintf(config_file, "BG_COLOR_BUT = 41\n");
-      fprintf(config_file, "#Show README on startup\n");
-      fprintf(config_file, "SHOW_README\n");
-      fprintf(config_file, "#GnuDOS Level\n");
-      fprintf(config_file, "GNU_DOS_LEVEL = 1\n");
-      fprintf(config_file, "#Auto-indentation\n");
-      fprintf(config_file, "AUTO_INDENT = 1\n");
-      fclose(config_file);
-      printf("Finished writing default values to ~/.mino.conf\n");
-      exit(0);
-    } 
-    else if(strcmp(argv[i], "--help") == 0 ||
-	      strcmp(argv[i], "-h") == 0) 
-    {//if #3
-      printf("\nCommand line help for mino under GNU/Linux\n");
-      printf("Copyright (C) Mohammed Isam 2014\n");
-      printf("This is a GNU software, part of the GnuDOS package\n\n");
-      printf("Usage: %s [filename][-options]\n\n", argv[0]);
-      printf("Options:\n\n");
-      printf("\tfilename: File to be opened by mino (optional)\n");
-      printf("\t--reset-config\tReset program configuration. Writes default\n");
-      printf("\t              \tvalues to .mino.conf file\n");
-      printf("\t--help, -h    \tShow this command line help\n");
-      printf("\t--version, -v \tShow program version\n");
-      printf("\t--levelX, -lX \tSet the experience level, where X is 1-6\n");
-      printf("For more information, see 'info mino' or the README file in mino help menu\n");
-      printf("\n");
-      exit(0);
-    }
-    else if(strcmp(argv[i], "--version") == 0
-	    || strcmp(argv[i], "-v") == 0) 
-    {//if #3
-      printf("Mino version 1.2\n");
-      exit(0);
-    }
-    else 
-    {
-	NEW_FILE = 0; FILE_STATE = OPENED;
-	open_file_at_startup = 1;
-      //return;
-    }//end if #2
-  }//end if #1
-  */
 }//end parseLineArgs()
 
 void init_error() 
@@ -390,6 +282,7 @@ void init()
       return;
     }
     config_file_name = (char *) malloc(strlen(pass->pw_dir)+11);
+    if(!config_file_name) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
     strcpy(config_file_name, pass->pw_dir);
     strcat(config_file_name, "/");
     strcat(config_file_name, ".mino.conf");
@@ -485,12 +378,13 @@ void init()
    }
    else if (strstr(buf, "TAB_CHARS")) 
    {
-     //TAB_CHARS = atoi((buf, strchr(buf, '=')+2));
      TAB_CHARS = atoi(strchr(buf, '=')+2);
+     if(TAB_CHARS < 1 || TAB_CHARS > SCREEN_W-3) TAB_CHARS = 8;
    }
    else if (strstr(buf, "DEFAULT_TITLE")) 
    {
      DEFAULT_TITLE = (char *)malloc(MAX_FILE_NAME_LEN);
+     if(!DEFAULT_TITLE) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
      strcpy(DEFAULT_TITLE, strchr(buf, '=')+2);
      DEFAULT_TITLE[strlen(DEFAULT_TITLE)-1] = '\0';
    }
@@ -511,62 +405,50 @@ void init()
    }
    else if (strstr(buf, "FG_COLOR_WIN")) 
    {
-     //FG_COLOR[COLOR_WINDOW] = atoi((buf, strchr(buf, '=')+2));
      FG_COLOR[COLOR_WINDOW] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "FG_COLOR_HLT")) 
    {
-     //FG_COLOR[COLOR_HIGHLIGHT_TEXT] = atoi((buf, strchr(buf, '=')+2));
      FG_COLOR[COLOR_HIGHLIGHT_TEXT] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "FG_COLOR_MBAR")) 
    {
-     //FG_COLOR[COLOR_MENU_BAR] = atoi((buf, strchr(buf, '=')+2));
      FG_COLOR[COLOR_MENU_BAR] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "FG_COLOR_SBAR")) 
    {
-     //FG_COLOR[COLOR_STATUS_BAR] = atoi((buf, strchr(buf, '=')+2));
      FG_COLOR[COLOR_STATUS_BAR] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "FG_COLOR_HBUT")) 
    {
-     //FG_COLOR[COLOR_HBUTTONS] = atoi((buf, strchr(buf, '=')+2));
      FG_COLOR[COLOR_HBUTTONS] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "FG_COLOR_BUT")) 
    {
-     //FG_COLOR[COLOR_BUTTONS] = atoi((buf, strchr(buf, '=')+2));
      FG_COLOR[COLOR_BUTTONS] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "BG_COLOR_WIN")) 
    {
-     //BG_COLOR[COLOR_WINDOW] = atoi((buf, strchr(buf, '=')+2));
      BG_COLOR[COLOR_WINDOW] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "BG_COLOR_HLT")) 
    {
-     //BG_COLOR[COLOR_HIGHLIGHT_TEXT] = atoi((buf, strchr(buf, '=')+2));
      BG_COLOR[COLOR_HIGHLIGHT_TEXT] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "BG_COLOR_MBAR")) 
    {
-     //BG_COLOR[COLOR_MENU_BAR] = atoi((buf, strchr(buf, '=')+2));
      BG_COLOR[COLOR_MENU_BAR] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "BG_COLOR_SBAR")) 
    {
-     //BG_COLOR[COLOR_STATUS_BAR] = atoi((buf, strchr(buf, '=')+2));
      BG_COLOR[COLOR_STATUS_BAR] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "BG_COLOR_HBUT")) 
    {
-     //BG_COLOR[COLOR_HBUTTONS] = atoi((buf, strchr(buf, '=')+2));
      BG_COLOR[COLOR_HBUTTONS] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "BG_COLOR_BUT")) 
    {
-     //BG_COLOR[COLOR_BUTTONS] = atoi((buf, strchr(buf, '=')+2));
      BG_COLOR[COLOR_BUTTONS] = atoi(strchr(buf, '=')+2);
    }
    else if (strstr(buf, "SHOW_README")) 
@@ -597,39 +479,41 @@ void init()
  firstVisDir = 0;
  selectedDir = -1;
  totalDirs = 0;
- 
+
  MAX_DIR_NAME_LEN = (SCREEN_W/2)-8;
  MAX_MSG_BOX_W = SCREEN_W - 4;
  MAX_MSG_BOX_H = SCREEN_H - 4;
  
- //dirs = (char *) malloc(MAX_DIR_NAME_LEN*MAXDIRS);
  for(i = 0; i < MAXDIRS; i++) 
  {
    dirs[i] = (char *) malloc(MAX_DIR_NAME_LEN);
+   if(!dirs[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  }
  for(i = 0; i < MAXFILES; i++) 
  {
-   files[i] = (char *) malloc(MAX_FILE_NAME_LEN);
+   files[i] = (char *) malloc(MAX_DIR_NAME_LEN);
+   if(!files[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  }
  
  if(WRAP_LINES) MAX_CHARS_PER_LINE = SCREEN_W-2;
  
- //totalCharsInLine = malloc(sizeof(int)*MAX_LINES);
- //lines = malloc(MAX_CHARS_PER_LINE*MAX_LINES);
-
  if(!DEFAULT_TITLE) 
  { 
    DEFAULT_TITLE = (char *) malloc(MAX_FILE_NAME_LEN*4);
+   if(!DEFAULT_TITLE) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
    strcpy(DEFAULT_TITLE, "untitled");
  }
  if(!documentTitle) documentTitle = (char *) malloc(MAX_FILE_NAME_LEN*4);
+ if(!documentTitle) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  open_file_name = (char *) malloc(MAX_FILE_NAME_LEN*4);
+ if(!open_file_name) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  if(NEW_FILE) 
  {
    strcpy(documentTitle, DEFAULT_TITLE);
    for(i = 0; i < MAX_LINES; i++) 
    {
     lines[i] = (char *) malloc((MAX_CHARS_PER_LINE+TAB_CHARS)*4);
+    if(!lines[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
     strcpy(lines[i], "\0");
     totalCharsInLine[i] = 0;
     LINE_IS_LINKED[i] = 0;
@@ -647,15 +531,17 @@ void init()
  selectedCharCarry = 0;
  totalVisLines = SCREEN_H-4;
  firstVisLine = 0;
- //WRAP_LINES = 1;//TRUE -- restric line width to (screen width-2)
+ //WRAP_LINES = 1;//TRUE -- restrict line width to (screen width-2)
  //TAB_CHARS = 8;	//define 8 spaces in a tab
  SELECTING = 0;
  CLIPBOARD_IS_EMPTY = 1;
  last_undo_action = -1;
  for(i = 0; i < MAX_UNDO_ACTIONS; i++) 
  {
-   undo_text[i] = (char *) malloc(MAX_CHARS_PER_LINE*10);
-   undo_text_replace[i] = (char *) malloc(MAX_CHARS_PER_LINE*10);
+   undo_text[i] = 0; //(char *) malloc(MAX_CHARS_PER_LINE*10);
+   //if(!undo_text[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
+   undo_text_replace[i] = 0; //(char *) malloc(MAX_CHARS_PER_LINE*10);
+   //if(!undo_text_replace[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  }
  
  if(!initTerminal()) 
@@ -664,8 +550,6 @@ void init()
    exit(1);
  }
  catchSignals();
- //AUTO_HIGHLIGHTING = 0;
- //loadKeywords(HIGHLIGHT_MODE);
 }
 
 /***************************************
@@ -761,13 +645,14 @@ void init_built_in_defaults()
  for(i = 0; i < MAXDIRS; i++) 
  {
    dirs[i] = (char *) malloc(MAX_DIR_NAME_LEN);
+   if(!dirs[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
+ }
+ for(i = 0; i < MAXFILES; i++) 
+ {
+   files[i] = (char *) malloc(MAX_DIR_NAME_LEN);
+   if(!files[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  }
  MAX_CHARS_PER_LINE = SCREEN_W-2;
- /*for(i = 0; i < MAX_LINES; i++) {
-   lines[i] = (char *) malloc(MAX_CHARS_PER_LINE+TAB_CHARS);
-   strcpy(lines[i], "\0");
-   totalCharsInLine[i] = 0;
- }*/
  selectedLine = 0;
  selectedChar = 0;
  selectedCharCarry = 0;
@@ -777,21 +662,20 @@ void init_built_in_defaults()
  WRAP_LINES = 1;//TRUE -- restric line width to (screen width-2)
  TAB_CHARS = 8;	//define 8 spaces in a tab
 
- /*if(!documentTitle) documentTitle = (char *) malloc(MAX_FILE_NAME_LEN);
- strcpy(documentTitle, "untitled");
- open_file_name = (char *) malloc(MAX_FILE_NAME_LEN);
- */
- 
  if(!documentTitle) documentTitle = (char *) malloc(MAX_FILE_NAME_LEN*4);
+ if(!documentTitle) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  if(!DEFAULT_TITLE) DEFAULT_TITLE = (char *) malloc(MAX_FILE_NAME_LEN*4);
+ if(!DEFAULT_TITLE) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  strcpy(DEFAULT_TITLE, "untitled");
  open_file_name = (char *) malloc(MAX_FILE_NAME_LEN*4);
+ if(!open_file_name) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  if(NEW_FILE) 
  {
    strcpy(documentTitle, DEFAULT_TITLE);
    for(i = 0; i < MAX_LINES; i++) 
    {
     lines[i] = (char *) malloc((MAX_CHARS_PER_LINE+TAB_CHARS)*4);
+    if(!lines[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
     strcpy(lines[i], "\0");
     totalCharsInLine[i] = 0;
     LINE_IS_LINKED[i] = 0;
@@ -815,8 +699,10 @@ void init_built_in_defaults()
  RECORDING_UNDO_ACTION = 0;
  for(i = 0; i < MAX_UNDO_ACTIONS; i++) 
  {
-   undo_text[i] = (char *) malloc(MAX_CHARS_PER_LINE*10);
-   undo_text_replace[i] = (char *) malloc(MAX_CHARS_PER_LINE*10);
+   undo_text[i] = 0;//(char *) malloc(MAX_CHARS_PER_LINE*10);
+   //if(!undo_text[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
+   undo_text_replace[i] = 0;//(char *) malloc(MAX_CHARS_PER_LINE*10);
+   //if(!undo_text_replace[i]) { fprintf(stderr, "Insufficient memory\n"); exit(1); }
  }
 
  if(!initTerminal()) 
