@@ -29,6 +29,8 @@
 #include <stdlib.h>
 #include <string.h>	//for memset()
 
+unsigned short mask[] = {192, 224, 240};
+
 static struct termios tty_attr_old;
 static int old_keyboard_mode;
 
@@ -267,103 +269,117 @@ char *getKeyUnderX()
     {
       c = getchar();
       if(c < 0) continue;
+      //printf("%c[%d]",  c, c);
+      //fflush(stdout);
       //bytes = 0;
       if ((c & mask[0]) == mask[0]) bytes++;
       if ((c & mask[1]) == mask[1]) bytes++;
       if ((c & mask[2]) == mask[2]) bytes++;
     
       //fprintf(stdout, "..%c, %d..", c, c);
- 	      if(c == 127) { uc[0] = BACKSPACE_KEY; }
- 	      else if(c == 31) { 
+      if(c == 127)
+      { 
+	uc[0] = BACKSPACE_KEY;
+      }
+      else if(c == 31)
+      { 
 	   c = getchar();
-		CTRL=1; uc[0] = '/'; }
- 	      else if(c == 26) { CTRL=1; uc[0] = 'z'; }
- 	      else if(c == 25) { CTRL=1; uc[0] = 'y'; }
- 	      else if(c == 03) { CTRL=1; uc[0] = 'c'; }
- 	      else if(c == 24) { CTRL=1; uc[0] = 'x'; }
- 	      else if(c == 22) { CTRL=1; uc[0] = 'v'; }
- 	      else if(c == 15) { CTRL=1; uc[0] = 'o'; }
- 	      else if(c == 11) { CTRL=1; uc[0] = 'k'; }
- 	      else if(c == 16) { CTRL=1; uc[0] = 'p'; }
- 	      else if(c == 07) { CTRL=1; uc[0] = 'g'; }
- 	      else if(c == 06) { CTRL=1; uc[0] = 'f'; }
- 	      else if(c == 05) { CTRL=1; uc[0] = 'e'; }
- 	      else if(c == 04) { CTRL=1; uc[0] = 'd'; }
- 	      else if(c == 02) { CTRL=1; uc[0] = 'b'; }
- 	      else if(c == 01) { CTRL=1; uc[0] = 'a'; }
- 	      else if(c == 19) { CTRL=1; uc[0] = 's'; }
- 	      else if(c == 18) { CTRL=1; uc[0] = 'r'; }
- 	      else if(c == 14) { CTRL=1; uc[0] = 'n'; }
- 	      else if(c == 17) { CTRL=1; uc[0] = 'q'; }
- 	      else if(c == 23) { CTRL=1; uc[0] = 'w'; }
- 	      else if(c == 00) { CTRL=1; uc[0] = SPACE_KEY; }
- 	      else if(c == 32) 
-	      {	//the SPACEBAR is pressed
-		uc[0] = SPACE_KEY;
-	      }
-	      else if(c == 10 || c == 13) 
-	      {	//the ENTER is pressed
-		uc[0] = ENTER_KEY;
-	      }
-	      else if(c == 9) 
-	      {	//the TAB is pressed
-		uc[0] = TAB_KEY;
-	      }
-	  else if(c == 27) 
-	  {	//ESC key pressed -- maybe starting an escape sequence??
-	   c = getchar();
-	   //fprintf(stdout, "..%c, %d..", c, c);
-	   if((c == 'f') || (c == 'F')) { ALT=1; uc[0] = 'f'; }
-	   else if((c == 'e') || (c == 'E')) { ALT=1; uc[0] = 'e'; }
-	   else if((c == 'h') || (c == 'H')) { ALT=1; uc[0] = 'h'; }
-	   else if((c == 'o') || (c == 'O')) { ALT=1; uc[0] = 'o'; }
-	   else if((c == 'b') || (c == 'B')) { ALT=1; uc[0] = 'b'; }
-	   else if((c == 'v') || (c == 'V')) { ALT=1; uc[0] = 'v'; }
-	   else if((c == 'd') || (c == 'D')) { ALT=1; uc[0] = 'd'; }
-	   else if(c == 127) { ALT=1; uc[0] = BACKSPACE_KEY; }
-	   else if(c == 91) 
-	   {	//yep -- this is the left bracket '[' -- so there is something coming
+	   CTRL=1; uc[0] = '/';
+      }
+      else if(c == 26) { CTRL=1; uc[0] = 'z'; }
+      else if(c == 25) { CTRL=1; uc[0] = 'y'; }
+      else if(c == 03) { CTRL=1; uc[0] = 'c'; }
+      else if(c == 24) { CTRL=1; uc[0] = 'x'; }
+      else if(c == 22) { CTRL=1; uc[0] = 'v'; }
+      else if(c == 15) { CTRL=1; uc[0] = 'o'; }
+      else if(c == 11) { CTRL=1; uc[0] = 'k'; }
+      else if(c == 16) { CTRL=1; uc[0] = 'p'; }
+      else if(c == 07) { CTRL=1; uc[0] = 'g'; }
+      else if(c == 06) { CTRL=1; uc[0] = 'f'; }
+      else if(c == 05) { CTRL=1; uc[0] = 'e'; }
+      else if(c == 04) { CTRL=1; uc[0] = 'd'; }
+      else if(c == 02) { CTRL=1; uc[0] = 'b'; }
+      else if(c == 01) { CTRL=1; uc[0] = 'a'; }
+      else if(c == 19) { CTRL=1; uc[0] = 's'; }
+      else if(c == 18) { CTRL=1; uc[0] = 'r'; }
+      else if(c == 14) { CTRL=1; uc[0] = 'n'; }
+      else if(c == 17) { CTRL=1; uc[0] = 'q'; }
+      else if(c == 23) { CTRL=1; uc[0] = 'w'; }
+      else if(c == 00) { CTRL=1; uc[0] = SPACE_KEY; }
+      else if(c == 32) 
+      {	//the SPACEBAR is pressed
+	uc[0] = SPACE_KEY;
+      }
+      else if(c == 10 || c == 13) 
+      {	//the ENTER is pressed
+	uc[0] = ENTER_KEY;
+      }
+      else if(c == 9) 
+      {	//the TAB is pressed
+	uc[0] = TAB_KEY;
+      }
+      else if(c == 27) 
+      {	//ESC key pressed -- maybe starting an escape sequence??
+	c = getchar();
+	//fprintf(stdout, "..%c, %d..", c, c);
+	if((c == 'f') || (c == 'F')) { ALT=1; uc[0] = 'f'; }
+	else if((c == 'e') || (c == 'E')) { ALT=1; uc[0] = 'e'; }
+	else if((c == 'h') || (c == 'H')) { ALT=1; uc[0] = 'h'; }
+	else if((c == 'o') || (c == 'O')) 
+	{ 
+	  c = getchar();
+	  if(c == 72) { uc[0] = HOME_KEY; }
+	  else if(c == 70) { uc[0] = END_KEY; }
+	  else { ALT=1; uc[0] = 'o'; }	//if c == 111
+	}
+	else if((c == 'b') || (c == 'B')) { ALT=1; uc[0] = 'b'; }
+	else if((c == 'v') || (c == 'V')) { ALT=1; uc[0] = 'v'; }
+	else if((c == 'd') || (c == 'D')) { ALT=1; uc[0] = 'd'; }
+	else if(c == 127) { ALT=1; uc[0] = BACKSPACE_KEY; }
+	else if(c == 91) 
+	{	//yep -- this is the left bracket '[' -- so there is something coming
+	  c = getchar();
+	  //fprintf(stdout, "..%c, %d..", c, c);
+	  if(c == 65) { uc[0] = UP_KEY; }
+	  else if(c == 66) { uc[0] = DOWN_KEY; }
+	  else if(c == 67) { uc[0] = RIGHT_KEY; }
+	  else if(c == 68) { uc[0] = LEFT_KEY; }
+	  else if(c == 72) { uc[0] = HOME_KEY; }
+	  else if(c == 70) { uc[0] = END_KEY; }
+	  else if(c == 54) { c = getchar(); uc[0] = PGDOWN_KEY; }
+	  else if(c == 53) { c = getchar(); uc[0] = PGUP_KEY; }
+	  else if(c == 50) { c = getchar(); uc[0] = INS_KEY; }
+	  else if(c == 51) 
+	  { 
+	    c = getchar(); 
+	    if(c == 126) uc[0] = DEL_KEY;
+	    else if(c == 59) 
+	    { 
+	      c = getchar();
+	      c = getchar();
+	      //if(c == 53) { c = getchar; 
+	      CTRL = 1; uc[0] = DEL_KEY; //}
+	    }
+	  }
+	  else if(c == 49)
+	  { //this is CTRL-something
+	    CTRL = 1; 
 	    c = getchar();
 	    //fprintf(stdout, "..%c, %d..", c, c);
-	      if(c == 65) { uc[0] = UP_KEY; }
-	      else if(c == 66) { uc[0] = DOWN_KEY; }
-	      else if(c == 67) { uc[0] = RIGHT_KEY; }
-	      else if(c == 68) { uc[0] = LEFT_KEY; }
-	      else if(c == 72) { uc[0] = HOME_KEY; }
-	      else if(c == 70) { uc[0] = END_KEY; }
-	      else if(c == 54) { c = getchar(); uc[0] = PGDOWN_KEY; }
-	      else if(c == 53) { c = getchar(); uc[0] = PGUP_KEY; }
-	      else if(c == 50) { c = getchar(); uc[0] = INS_KEY; }
-	      else if(c == 51) 
-	      { 
-		c = getchar(); 
-		if(c == 126) uc[0] = DEL_KEY;
-		else if(c == 59) 
-		{ 
-		  c = getchar();
-		  //if(c == 53) { c = getchar; 
-		  CTRL = 1; uc[0] = DEL_KEY; //}
-		}
-	      }
-	      else if(c == 49)
-	      { //this is CTRL-something
-		CTRL = 1; 
+	    if(c == 59) 
+	    {
+	      c = getchar();
+	      if(c == 53 || c == 50)
+	      {
 		c = getchar();
-		//fprintf(stdout, "..%c, %d..", c, c);
-		if(c == 59) 
-		{
-		  c = getchar();
-		  if(c == 53) 
-		  {
-		    c = getchar();
-		    if(c == 67) uc[0] = RIGHT_KEY; 
-		    else if(c == 68) uc[0] = LEFT_KEY; 
-		    else if(c == 65) uc[0] = UP_KEY; 
-		    else if(c == 66) uc[0] = DOWN_KEY; 
-		    else if(c == 72) uc[0] = HOME_KEY; 
-		    else if(c == 70) uc[0] = END_KEY; 
-		  }
-		}//end if
+		if(c == 67) uc[0] = RIGHT_KEY; 
+		else if(c == 68) uc[0] = LEFT_KEY; 
+		else if(c == 65) uc[0] = UP_KEY; 
+		else if(c == 66) uc[0] = DOWN_KEY; 
+		else if(c == 72) uc[0] = HOME_KEY; 
+		else if(c == 70) uc[0] = END_KEY; 
+	      }
+	    }//end if
 	  }//end if(c == 49)
 	}//end if(c == 91)
 	else uc[0] = ESC_KEY;
@@ -375,7 +391,8 @@ char *getKeyUnderX()
       {
 	uc[0] = c;
 	int i = 0;
-	while(i < bytes) {
+	while(i < bytes)
+	{
 	  c = getchar();
 	  uc[++i] = c;
 	}//end while
