@@ -52,14 +52,11 @@ void showFileMenu(int visible)
   int endme = 0;
   if(visible == YES) 
   {	//show the file menu
-    //activeWindow = FILE_MENU;
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
     drawBox(2, 2, 9, 19, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-    //setScreenColors(BLUE, BGWHITE);
     fprintf(stdout, "\e[3;3H%s", fileMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-    //setScreenColors(WHITE, BGBLUE);
     for(i = 1; i < fTotal; i++) 
     {
       fprintf(stdout, "\e[%d;3H%s", i+3, fileMenu[i]);
@@ -79,7 +76,6 @@ void showFileMenu(int visible)
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  fflush(stdout);
 	  endme = 1;
@@ -92,12 +88,10 @@ do_esc:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_up:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;3H%s", fSelect+3, fileMenu[fSelect]);	//clear last selection
 	  fSelect--;
 	  if(fSelect < 0) fSelect = fTotal-1;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;3H%s", fSelect+3, fileMenu[fSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;18H", fSelect+3);
 	  fflush(stdout);
@@ -110,12 +104,10 @@ do_up:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_down:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;3H%s", fSelect+3, fileMenu[fSelect]);	//clear last selection
 	  fSelect++;
 	  if(fSelect >= fTotal) fSelect = 0;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;3H%s", fSelect+3, fileMenu[fSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;18H", fSelect+3);
 	  fflush(stdout);
@@ -124,7 +116,6 @@ do_down:
 	  if(ALT)
 	  {
 	    setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	    //setScreenColors(WHITE, BGBLUE);
 	    refreshView();
 	    return;
 	  }
@@ -137,7 +128,6 @@ do_down:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showEditMenu(YES);
 	  endme = 1;
@@ -153,7 +143,6 @@ do_right:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showHelpMenu(YES);
 	  endme = 1;
@@ -195,12 +184,9 @@ do_left:
 	  }
 	  
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  endme = 1;
 	  break;
-	  //return;
-	  break;
       }
     }
   } 
@@ -228,14 +214,11 @@ void showEditMenu(int visible)
   int endme = 0;
   if(visible == YES) 
   {	//show the file menu
-    //activeWindow = Mino_MENU;
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
     drawBox(2, 8, 13, 30, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-    //setScreenColors(BLUE, BGWHITE);
     fprintf(stdout, "\e[3;9H%s", editMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-    //setScreenColors(WHITE, BGBLUE);
     for(i = 1; i < eTotal; i++) 
     {
       fprintf(stdout, "\e[%d;9H%s", i+3, editMenu[i]);
@@ -255,7 +238,6 @@ void showEditMenu(int visible)
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  endme = 1;
 	  break;
@@ -267,12 +249,10 @@ do_esc:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_up:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;9H%s", eSelect+3, editMenu[eSelect]);	//clear last selection
 	  eSelect--;
 	  if(eSelect < 0) eSelect = eTotal-1;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;9H%s", eSelect+3, editMenu[eSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;29H", eSelect+3);
 	  fflush(stdout);
@@ -285,12 +265,10 @@ do_up:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_down:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;9H%s", eSelect+3, editMenu[eSelect]);	//clear last selection
 	  eSelect++;
 	  if(eSelect >= eTotal) eSelect = 0;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;9H%s", eSelect+3, editMenu[eSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;29H", eSelect+3);
 	  fflush(stdout);
@@ -298,7 +276,6 @@ do_down:
 	case('e'):	//hide the file menu and return control to main program
 	  if(!ALT) break;
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  return;
 	  break;
@@ -322,7 +299,6 @@ do_left:
 	case('h'):	//navigate to Help menu
 	  if(!ALT) break;
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showHelpMenu(YES);
 	  endme = 1;
@@ -334,7 +310,6 @@ do_left:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showOptionsMenu(YES);
 	  endme = 1;
@@ -351,7 +326,6 @@ do_right:
 	  if(eSelect == 8) editMenu_Replace();
 	  if(eSelect == 9) editMenu_ToggleSelectMode();
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  endme = 1;
 	  break;
@@ -384,14 +358,11 @@ void showOptionsMenu(int visible)
   int endme = 0;
   if(visible == YES) 
   {	//show the file menu
-    //activeWindow = HELP_MENU;
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
     drawBox(2, 14, 7, 31, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-    //setScreenColors(BLUE, BGWHITE);
     fprintf(stdout, "\e[3;15H%s", optionsMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-    //setScreenColors(WHITE, BGBLUE);
     for(i = 1; i < hTotal; i++) fprintf(stdout, "\e[%d;15H%s", i+3, optionsMenu[i]);
     fprintf(stdout, "\e[3;30H");
     fflush(stdout);
@@ -408,7 +379,6 @@ void showOptionsMenu(int visible)
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  endme = 1;
 	  break;
@@ -420,12 +390,10 @@ do_esc:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_up:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+3, optionsMenu[oSelect]);	//clear last selection
 	  oSelect --;
 	  if(oSelect < 0) oSelect = oTotal-1;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+3, optionsMenu[oSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;30H", oSelect+3);
 	  fflush(stdout);
@@ -438,12 +406,10 @@ do_up:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_down:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+3, optionsMenu[oSelect]);	//clear last selection
 	  oSelect++;
 	  if(oSelect >= oTotal) oSelect = 0;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+3, optionsMenu[oSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;30H", oSelect+3);
 	  fflush(stdout);
@@ -451,7 +417,6 @@ do_down:
 	case('o'):	//hide the file menu and return control to main program
 	  if(!ALT) break;
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  return;
 	  break;
@@ -462,7 +427,6 @@ do_down:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showHelpMenu(YES);
 	  endme = 1;
@@ -471,7 +435,6 @@ do_right:
 	  if(CTRL && GNU_DOS_LEVEL > 1) goto do_right;
 	  if(!ALT) break;
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showFileMenu(YES);
 	  endme = 1;
@@ -487,7 +450,6 @@ do_right:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showEditMenu(YES);
 	  endme = 1;
@@ -519,12 +481,9 @@ do_left:
 	    optionsMenu_Reset_Config();
 	  }
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
-	  //refreshBottomView();
 	  endme = 1;
 	  break;
-	  //return;
       }
     }
   } 
@@ -553,14 +512,11 @@ void showHelpMenu(int visible)
   int endme = 0;
   if(visible == YES) 
   {	//show the file menu
-    //activeWindow = HELP_MENU;
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
     drawBox(2, 23, 7, 40, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-    //setScreenColors(BLUE, BGWHITE);
     fprintf(stdout, "\e[3;24H%s", helpMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-    //setScreenColors(WHITE, BGBLUE);
     for(i = 1; i < hTotal; i++) fprintf(stdout, "\e[%d;24H%s", i+3, helpMenu[i]);
     fprintf(stdout, "\e[3;39H");
     fflush(stdout);
@@ -577,7 +533,6 @@ void showHelpMenu(int visible)
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  endme = 1;
 	  break;
@@ -589,12 +544,10 @@ do_esc:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_up:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;24H%s", hSelect+3, helpMenu[hSelect]);	//clear last selection
 	  hSelect--;
 	  if(hSelect < 0) hSelect = hTotal-1;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;24H%s", hSelect+3, helpMenu[hSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;39H", hSelect+3);
 	  fflush(stdout);
@@ -607,12 +560,10 @@ do_up:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_down:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;24H%s", hSelect+3, helpMenu[hSelect]);	//clear last selection
 	  hSelect++;
 	  if(hSelect >= hTotal) hSelect = 0;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;24H%s", hSelect+3, helpMenu[hSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;39H", hSelect+3);
 	  fflush(stdout);
@@ -620,7 +571,6 @@ do_down:
 	case('h'):	//hide the file menu and return control to main program
 	  if(!ALT) break;
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  return;
 	  break;
@@ -632,7 +582,6 @@ do_down:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showFileMenu(YES);
 	  endme = 1;
@@ -640,7 +589,6 @@ do_right:
 	case('e'):	//navigate to Mino menu
 	  if(!ALT) break;
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showEditMenu(YES);
 	  endme = 1;
@@ -656,7 +604,6 @@ do_right:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  refreshView();
 	  showOptionsMenu(YES);
 	  endme = 1;
@@ -672,7 +619,7 @@ do_left:
 	  } 
 	  else if(hSelect == 3) 
 	  {		//show About msgbox
-	    char *x = "Mino for GNU/Linux.\nDeveloped by Mohammed Isam,\n2014.";
+	    char *x = "Mino for GNU/Linux.\nDeveloped by Mohammed Isam,\n2014, 2015.";
 	    msgBox(x, OK, INFO);
 	  } 
 	  else if(hSelect == 2) 
@@ -683,7 +630,7 @@ do_left:
 	    fprintf(stdout, "\e[6;9HALT+F: Open File menu");
 	    fprintf(stdout, "\e[7;9HALT+E: Open Mino menu");
 	    fprintf(stdout, "\e[8;9HALT+O: Open Options menu");
-	    fprintf(stdout, "\e[9;7HALT+H: Open Help menu");
+	    fprintf(stdout, "\e[9;9HALT+H: Open Help menu");
 	    fprintf(stdout, "\e[11;7HShortcut keys:");
 	    fprintf(stdout, "\e[12;9HCTRL+O: Open file dialog");
 	    fprintf(stdout, "\e[13;9HCTRL+S: Save file");
@@ -696,7 +643,7 @@ do_left:
 	    fprintf(stdout, "\e[20;9HCTRL+A: Select All");
 	    fprintf(stdout, "\e[21;9HCTRL+F: Find");
 	    fprintf(stdout, "\e[22;9HCTRL+R: Find & Replace");
-	    fprintf(stdout, "\e[24;7HPress any key to continue..");
+	    fprintf(stdout, "\e[23;7HPress any key to continue..");
 	    fflush(stdout);
 	    do {;} while(!getKey());
 	  }
@@ -717,167 +664,8 @@ do_left:
 }
 
 
-////////////////////////////////////////////////////////
-//shows a window with KEYBINDINGS file as its contents//
-////////////////////////////////////////////////////////
-void showKeybindings() 
-{
-  int x = 2;
-  int y = 2;
-  int w = SCREEN_W-1;
-  int h = SCREEN_H-1;
-  FILE *KEYBINDINGS;
-  char buf[4096*4];	//buffer to hold data
-  int buf_len = 0;
-  
-  if(!(KEYBINDINGS = fopen("/usr/share/doc/gnudos/mino/keybindings", "r"))) 
-  {
-   if(!(KEYBINDINGS = fopen("/usr/local/share/doc/gnudos/mino/keybindings", "r"))) 
-   {
-    msgBox("Failed to open KEYBINDINGS file!.", OK, ERROR);
-    //scanDir(cwd);
-    refreshView();
-    return;
-   }
-  }
-  
-  if(!(buf_len = fread(buf, sizeof(char), sizeof(buf), KEYBINDINGS))) 
-  {
-    msgBox("Failed to read the KEYBINDINGS file!.", OK, ERROR);
-    //scanDir(cwd);
-    refreshView();
-    return;
-  }
-  
-  drawBox(x, y, h, w, " KEYBINDINGS ", YES);
-  
-  y++; x++;
-  int i = 0, j = x;
-  int l = y;
-  int lineStart[60];
-  int firstVisLine = 0;
-  int cnt = 0;
-  char *ch = (char *)malloc(5);
-  char moreLines = 1;	//used as boolean to indicate if still more lines
-  lineStart[0] = 0;
-  
-  fprintf(stdout, "\e[%d;%dH", j, l);
-  //for(i = 0; i < buf_len; i++) {
-  while(j < (h-x+2)) 
-  {
-    if(buf[i] == EOL) 
-    { 
-      lineStart[++cnt] = i+1;
-      l=y;
-      fprintf(stdout, "\e[%d;%dH", ++j, l); 
-    } 
-    else 
-    { 
-      if(l > (w-y+2)) 
-      {
-	lineStart[++cnt] = i;
-	l=y;
-	fprintf(stdout, "\e[%d;%dH", ++j, l);
-      } putchar(buf[i]); l++;
-    }
-    if(++i >= buf_len) break;
-  }
-  
-  fflush(stdout);
-  //if the output chars are less than the buffer length,
-  //it means there are more lines to the output.
-  if(i <= buf_len) 
-  {
-    goto readMore;
-  } 
-  else 
-  {
-    do {;} while(!getKey());
-    goto end;
-    return;
-  }
-  
-readMore:
-  while(1) 
-  {
-    ch = getKey();
-    switch(ch[0]) 
-    {
-      case('g'):
-	if(GNU_DOS_LEVEL < 3) break;
-	if(!CTRL) break;
-	goto do_esc;
-      case(ESC_KEY):
-	if(GNU_DOS_LEVEL > 2) break;
-do_esc:
-      case(ENTER_KEY):
-      case(SPACE_KEY):
-	goto end;
-	break;
-      case('p'):
-	if(GNU_DOS_LEVEL < 2) break;
-	if(!CTRL) break;
-	goto do_up;
-      case(UP_KEY):
-	if(GNU_DOS_LEVEL > 1) break;
-do_up:
-	if(firstVisLine == 0) break;	//can't go up--first line already
-	firstVisLine--;
-	break;
-      case('n'):
-	if(GNU_DOS_LEVEL < 2) break;
-	if(!CTRL) break;
-	goto do_down;
-      case(DOWN_KEY):
-	if(GNU_DOS_LEVEL > 1) break;
-do_down:
-	if(!moreLines) break;	//reached last line
-	firstVisLine++;
-	break;
-    }	//end of switch
-    //redraw the box with its contents
-    drawBox(x-1, y-1, h, w, " KEYBINDINGS ", YES);
-    j = x; l = y; cnt = firstVisLine; i = lineStart[firstVisLine];
-    
-    fprintf(stdout, "\e[%d;%dH", j, l);
-    while(j < (h-x+2)) 
-    {
-     if(buf[i] == EOL) 
-     { 
-      lineStart[++cnt] = i+1;
-      l=y;
-      fprintf(stdout, "\e[%d;%dH", ++j, l); 
-     } 
-     else 
-     { 
-      if(l > (w-y+2)) 
-      {
-	lineStart[++cnt] = i;
-	l=y;
-	fprintf(stdout, "\e[%d;%dH", ++j, l);
-      } putchar(buf[i]); l++;
-     }
-     if(++i >= buf_len) break;
-    }	//end of inner while
-    if(i <= buf_len) { moreLines = 1; }
-    else { moreLines = 0; }
-    fflush(stdout);
 
-  }	//end of outer while
-end:
-  fclose(KEYBINDINGS);
-  //free(buf);
-  drawMenuBar(2, 2, SCREEN_W-2);							//draw main menu bar
-  //scanDir(cwd);
-  fflush(stdout);
-  return;
-}
-
-
-///////////////////////////////////////////////////
-//shows a window with README file as its contents//
-///////////////////////////////////////////////////
-void showReadMe() 
+void _do_show(int readme_keybindings)
 {
   int x = 2;
   int y = 2;
@@ -887,12 +675,28 @@ void showReadMe()
   char buf[4096*4];	//buffer to hold data
   int buf_len = 0;
   
-  if(!(README = fopen("/usr/share/doc/gnudos/mino/README", "r"))) 
+  char *file_name[] =
   {
-   if(!(README = fopen("/usr/local/share/doc/gnudos/mino/README", "r"))) 
+    "/usr/share/doc/gnudos/mino/README",
+    "/usr/local/share/doc/gnudos/mino/README",
+    "/usr/share/doc/gnudos/mino/keybindings",
+    "/usr/local/share/doc/gnudos/mino/keybindings"
+  };
+  char *title[] = { " README ", " KEYBINDINGS " };
+  char *err[] =
+  {
+    "Failed to read the README file!.",
+    "Failed to read the KEYBINDINGS file!."
+  };
+  /* choose file to open according to readme_keybindings flag */
+  char *file1 = readme_keybindings ? file_name[2] : file_name[0];
+  char *file2 = readme_keybindings ? file_name[3] : file_name[1];
+  
+  if(!(README = fopen(file1, "r"))) 
+  {
+   if(!(README = fopen(file2, "r"))) 
    {
-    msgBox("Failed to open README file!.", OK, ERROR);
-    //scanDir(cwd);
+    msgBox(err[readme_keybindings], OK, ERROR);
     refreshView();
     return;
    }
@@ -900,13 +704,12 @@ void showReadMe()
   
   if(!(buf_len = fread(buf, sizeof(char), sizeof(buf), README))) 
   {
-    msgBox("Failed to read the README file!.", OK, ERROR);
-    //scanDir(cwd);
+    msgBox(err[readme_keybindings], OK, ERROR);
     refreshView();
     return;
   }
   
-  drawBox(x, y, h, w, " README ", YES);
+  drawBox(x, y, h, w, title[readme_keybindings], YES);
   
   y++; x++;
   int i = 0, j = x;
@@ -919,7 +722,6 @@ void showReadMe()
   lineStart[0] = 0;
   
   fprintf(stdout, "\e[%d;%dH", j, l);
-  //for(i = 0; i < buf_len; i++) {
   while(j < (h-x+2)) 
   {
     if(buf[i] == EOL) 
@@ -993,7 +795,7 @@ do_down:
 	break;
     }	//end of switch
     //redraw the box with its contents
-    drawBox(x-1, y-1, h, w, " README ", YES);
+    drawBox(x-1, y-1, h, w, title[readme_keybindings], YES);
     j = x; l = y; cnt = firstVisLine; i = lineStart[firstVisLine];
     
     fprintf(stdout, "\e[%d;%dH", j, l);
@@ -1023,9 +825,23 @@ do_down:
   }	//end of outer while
 end:
   fclose(README);
-  //free(buf);
   drawMenuBar(2, 2, SCREEN_W-2);							//draw main menu bar
-  //scanDir(cwd);
   fflush(stdout);
   return;
 }
+
+////////////////////////////////////////////////////////
+//shows a window with KEYBINDINGS file as its contents//
+////////////////////////////////////////////////////////
+void showKeybindings() 
+{
+  _do_show(1);
+}
+
+///////////////////////////////////////////////////
+//shows a window with README file as its contents//
+///////////////////////////////////////////////////
+void showReadMe() 
+{
+  _do_show(0);
+}
\ No newline at end of file
