@@ -1,6 +1,6 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2014 (c)
+ *    Copyright 2014, 2015 (c)
  * 
  *    file: opensave.c
  *    This file is part of mino (Mino).
@@ -31,6 +31,10 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
+/* main.c */
+extern char *tmp;
+extern char *tmp2;
+
 void checkFileExtension();
 
 static int one(const struct dirent *unused) 
@@ -66,7 +70,8 @@ void findFullFileName(char *f)
     int cnt;
     for(cnt = 0; cnt < n; ++cnt) 
     {
-      lstat(eps[cnt]->d_name,&st);
+      int x = lstat(eps[cnt]->d_name,&st);
+      if(x == -1) { msgBox(strerror(errno), OK, ERROR); return; }
       if(S_ISDIR(st.st_mode)) 
       {
       } 
@@ -77,7 +82,6 @@ void findFullFileName(char *f)
 		strcpy(open_file_name, cwd);
 		strcat(open_file_name, "/");
 		strcat(open_file_name, eps[cnt]->d_name);
-	  //strcpy(open_file_name, eps[cnt]->d_name);
 	  return;
 	}
       } //end else
@@ -108,7 +112,8 @@ char *findFullDirName(char *d)
     int cnt;
     for(cnt = 0; cnt < n; ++cnt) 
     {
-      lstat(eps[cnt]->d_name,&st);
+      int x = lstat(eps[cnt]->d_name,&st);
+      if(x == -1) { msgBox(strerror(errno), OK, ERROR); return NULL; }
       if(S_ISDIR(st.st_mode)) 
       {
 	if(strstr(eps[cnt]->d_name, d))
@@ -119,6 +124,24 @@ char *findFullDirName(char *d)
   return NULL;
 }
 
+#define WRAP_DIR_OUTPUT			\
+({					\
+  if(pos >= totalDirs)			\
+  {					\
+    fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 	\
+    selectedDir-numVisDirs+3:selectedDir+3,				\
+    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,		\
+    files[pos-totalDirs]);						\
+  }									\
+  else 					\
+  {					\
+    fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 	\
+    selectedDir-numVisDirs+3:selectedDir+3,				\
+    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,		\
+    dirs[pos]);								\
+  }					\
+})
+
 /*****************************************
  * openSaveFile():
  * This function shows a dialog box that
@@ -147,6 +170,7 @@ int openSaveFile(OPEN_SAVE openSave, int showDialog)
       {
 	  char *tmp;
 	  tmp = (char *) malloc(strlen(documentTitle));
+	  if(!tmp) { msgBox("Insufficient memory", OK, ERROR); return 0; }
 	  strcpy(tmp, documentTitle);
 	  tmp[strrchr(documentTitle, '/')-documentTitle] = '\0';
 	  scanDir(tmp);
@@ -182,27 +206,14 @@ int openSaveFile(OPEN_SAVE openSave, int showDialog)
 	fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 	printf("File: ");
 	setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
-	fprintf(stdout, "\e[%d;%dH", h-2, y+strlen(inputName)+7);
+	printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
+	fprintf(stdout, "\e[%d;%dH", h-2, (int)(y+strlen(inputName)+7));
 	//reset cursor if it is an OPEN file dialog
 	if(openSave == OPEN) 
 	{//if #1
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
 	  int pos = firstVisDir+selectedDir;
-	  if(pos >= totalDirs) 
-	  {//if #2
-	    fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-		    selectedDir-numVisDirs+3:selectedDir+3,
-		    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-		    files[pos-totalDirs]);
-	  } 
-	  else 
-	  {
-	  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-		    selectedDir-numVisDirs+3:selectedDir+3,
-		    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-		    dirs[pos]);
-	  }//end if #2
+	  WRAP_DIR_OUTPUT;
 	}//end if #1
       fflush(stdout);
       /***************************************************/
@@ -253,7 +264,6 @@ do_del:
 		inputName[i] = inputName[i+1];
 	      inputName[i] = '\0';
 	      setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	      //setScreenColors(BLUE, BGWHITE);
 	      fprintf(stdout, "\e[%d;%dH", h-2, y+7);
 	      printf("%*s", w-y-9, " ");
 	      fprintf(stdout, "\e[%d;%dH", h-2, y+7);
@@ -281,7 +291,6 @@ do_del:
 	      inputName[i] = '\0';
 	      selChar--;
 	      setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	      //setScreenColors(BLUE, BGWHITE);
 	      fprintf(stdout, "\e[%d;%dH", h-2, y+7);
 	      printf("%*s", w-y-9, " ");
 	      fprintf(stdout, "\e[%d;%dH", h-2, y+7);
@@ -293,47 +302,18 @@ do_del:
 	  /***************************************************/
 	  /***************************************************/
 	  case(TAB_KEY):
-	    //if(openSave == SAVE) {
 	      sel = (sel == 0) ? 1 : 0;
 	      int pos = firstVisDir+selectedDir;
 	      if(sel == 1) 
 	      {//if #1
-		setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_WINDOW]);
-		//setScreenColors(GREEN, BGBLUE);
-		if(pos >= totalDirs) 
-		{//if #2
-		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    files[pos-totalDirs]);
-		} 
-		else 
-		{
-		  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    dirs[pos]);
-		}//if #2
+		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+		WRAP_DIR_OUTPUT;
 		fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 	      } 
 	      else 
 	      {
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
-		if(pos >= totalDirs) 
-		{//if #3
-		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    files[pos-totalDirs]);
-		} 
-		else 
-		{
-		  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    dirs[pos]);
-		}//if #3
+		WRAP_DIR_OUTPUT;
 	      }//if #1
 	    //} 
 	    break;
@@ -346,7 +326,6 @@ do_del:
 	  case(LEFT_KEY):
 	    if(GNU_DOS_LEVEL > 1) break;
 do_left:
-	    //if((openSave == SAVE) && (sel == 1)) {
 	    if(sel == 1) 
 	    {
 	      if(selChar == 0) break;
@@ -358,26 +337,11 @@ do_left:
 	    if(selectedDir >= numVisDirs) 
 	    {
 		int pos = firstVisDir+selectedDir;
-		setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_WINDOW]);
-		//setScreenColors(GREEN, BGBLUE);
-		if(pos >= totalDirs) 
-		{//if #5
-		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    files[pos-totalDirs]);
-		} 
-		else 
-		{
-		  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    dirs[pos]);
-		}//end if #5
+		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+		WRAP_DIR_OUTPUT;
 		selectedDir -= numVisDirs;
 		pos -= numVisDirs;
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
 		if(pos >= totalDirs) 
 		{//if #6
 		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
@@ -409,7 +373,7 @@ do_left:
 	      fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 	      printf("File: ");
 	      setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	      printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+	      printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 	      if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 	    fflush(stdout);
 	    break;
@@ -422,7 +386,6 @@ do_left:
 	  case(RIGHT_KEY):
 	    if(GNU_DOS_LEVEL > 1) break;
 do_right:
-	    //if((openSave == SAVE) && (sel == 1)) {
 	    if(sel == 1) 
 	    {
 	      if(selChar == strlen(inputName)) break;
@@ -435,26 +398,19 @@ do_right:
 	    if(selectedDir < numVisDirs) 
 	    {
 		int pos = firstVisDir+selectedDir;
-		setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_WINDOW]);
-		//setScreenColors(GREEN, BGBLUE);
-		if(pos >= totalDirs) 
-		{//if #5
-		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    files[pos-totalDirs]);
-		} 
-		else 
-		{
-		  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    dirs[pos]);
-		}//end if #5
+		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+		WRAP_DIR_OUTPUT;
 		selectedDir += numVisDirs;
 		pos += numVisDirs;
+		
+		if(pos >= (totalDirs+totalFiles))
+		{
+		  int h = (totalDirs+totalFiles)-pos-1;
+		  pos += h;  //h is negative, so add it in order to subtract it!!
+		  selectedDir += h;
+		}
+		
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
 		if(pos >= totalDirs) 
 		{//if #6
 		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
@@ -482,17 +438,12 @@ do_right:
 	      firstVisDir += numVisDirs;
 	      refreshDirView();
 	    } 
-	    //if SAVE dialog, redraw the input field
-	    //if(openSave == SAVE) {
 	      setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	      //setScreenColors(WHITE, BGBLUE);
 	      fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 	      printf("File: ");
 	      setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	      //setScreenColors(BLUE, BGWHITE);
-	      printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+	      printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 	      if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
-	    //}
 	    fflush(stdout);
 	    break;
 	  /***************************************************/
@@ -529,7 +480,7 @@ do_esc:
 	     	    fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 		    printf("File: ");
 		    setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		    printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+		    printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 		    if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 		    fflush(stdout);
 		    break; 
@@ -560,8 +511,8 @@ do_esc:
 	       char *tmp2;
 	       int i = (strrchr(cwd, '/')-cwd);
 	       tmp2 = (char *) malloc(i+2);
+	       if(!tmp2) { msgBox("Insufficient memory", OK, ERROR); return 0; }
 	       strncpy(tmp2, cwd, i);
-	       //strcat(tmp2, "\0");
 	       tmp2[i] = '/';
 	       tmp2[i+1] = '\0';
 	       scanDir(tmp2);
@@ -575,6 +526,7 @@ do_esc:
 		char *tmp2;
 		tmp2 = findFullDirName(dirs[firstVisDir+selectedDir]);
 		tmp = (char *) malloc(strlen(cwd)+strlen(tmp2+2));
+		if(!tmp) { msgBox("Insufficient memory", OK, ERROR); return 0; }
 		strcpy(tmp, cwd);
 		strcat(tmp, "/");
 		strcat(tmp, tmp2);
@@ -586,6 +538,7 @@ do_esc:
 	      {
 		tmp = (char *) malloc(strlen(cwd)+
 		      strlen(dirs[firstVisDir+selectedDir])+2);
+		if(!tmp) { msgBox("Insufficient memory", OK, ERROR); return 0; }
 		strcpy(tmp, cwd);
 		strcat(tmp, "/");
 		strcat(tmp, dirs[firstVisDir+selectedDir]);
@@ -598,12 +551,10 @@ do_esc:
 	     selectedDir = 0;
 	     refreshDirView();
 	     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	     //setScreenColors(WHITE, BGBLUE);
 	     fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 	     printf("File: ");
 	     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	     //setScreenColors(BLUE, BGWHITE);
-	     printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+	     printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 	     if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 	     fflush(stdout);
 	    } 
@@ -642,7 +593,7 @@ do_esc:
 	     	    fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 		    printf("File: ");
 		    setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		    printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+		    printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 		    if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 		    fflush(stdout);
 		    break;
@@ -665,9 +616,7 @@ do_esc:
 	  case(UP_KEY):
 	    if(GNU_DOS_LEVEL > 1) break;
 do_up:
-	    //if((openSave == SAVE) && (sel == 1)) break;
 	    if(sel == 1) break;
-	    //scanDir(".");
 	    if(selectedDir == 0) 
 	    {//if #1
 	      if(firstVisDir == 0) break;
@@ -678,25 +627,11 @@ do_up:
 	    else 
 	    {
 	      int pos = firstVisDir+selectedDir;
-		setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_WINDOW]);
-		//setScreenColors(GREEN, BGBLUE);
-		if(pos >= totalDirs) 
-		{//if #5
-		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    files[pos-totalDirs]);
-		} 
-		else 
-		{
-		  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    dirs[pos]);
-		}//end if #5
+		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+		WRAP_DIR_OUTPUT;
 		selectedDir--; pos--;
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
+		WRAP_DIR_OUTPUT;
 		if(pos >= totalDirs) 
 		{//if #6
 		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
@@ -720,12 +655,9 @@ do_up:
 		}//end if #6
 	      //}//end if #2
 	    }//end if #1
-	    //if SAVE dialog, redraw the input field
-	    //if(openSave == SAVE) {
 		fprintf(stdout, "\e[%d;%dH", h-2, y+7);
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
-		printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+		printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 		if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 	    //}
 	    fflush(stdout);
@@ -740,13 +672,11 @@ do_up:
 	  case(DOWN_KEY):
 	    if(GNU_DOS_LEVEL > 1) break;
 do_down:
-	    //if((openSave == SAVE) && (sel == 1)) break;
 	    if(sel == 1) break;
-	    //if((firstVisDir+selectedDir) >= (totalDirs+totalFiles)-1) break;
-	    //scanDir(".");
 	    if(selectedDir == (numVisDirs*2)-1) 
 	    {//if #1
-	      if((firstVisDir+(numVisDirs*2)) < (totalDirs+totalFiles)) {
+	      if((firstVisDir+(numVisDirs*2)) < (totalDirs+totalFiles))
+	      {
 		firstVisDir += numVisDirs;
 		selectedDir = numVisDirs;
 		refreshDirView();
@@ -754,12 +684,10 @@ do_down:
 	      if(openSave == SAVE) 
 	      {
 		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-		//setScreenColors(WHITE, BGBLUE);
 		fprintf(stdout, "\e[%d;%dH", h-2, y+1);
 		printf("File: ");
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
-		printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+		printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 		fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
 	      }
 		fflush(stdout);
@@ -770,25 +698,10 @@ do_down:
 	    {
 		int pos = firstVisDir+selectedDir;
 		if(pos >= (totalDirs+totalFiles-1)) break;
-		setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_WINDOW]);
-		//setScreenColors(GREEN, BGBLUE);
-		if(pos >= totalDirs) 
-		{//if #2
-		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    files[pos-totalDirs]);
-		} 
-		else 
-		{
-		  fprintf(stdout, "\x1b[%d;%dH[%s]", (selectedDir >= numVisDirs) ? 
-			    selectedDir-numVisDirs+3:selectedDir+3,
-			    (selectedDir >= numVisDirs) ? x+MAX_DIR_NAME_LEN+1:x+1,
-			    dirs[pos]);
-		}//end if #2
+		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+		WRAP_DIR_OUTPUT;
 		selectedDir++; pos++;
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
 		if(pos >= totalDirs) 
 		{//if #3
 		  fprintf(stdout, "\x1b[%d;%dH%s", (selectedDir >= numVisDirs) ? 
@@ -811,23 +724,16 @@ do_down:
 			    dirs[pos]);
 		}//end if #3
 	    }//end if #1
-	    //if SAVE dialog, redraw the input field
-	    //if(openSave == SAVE) {
 		fprintf(stdout, "\e[%d;%dH", h-2, y+7);
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
-		printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+		printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 		if(sel == 1) fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
-	    //}
 	    fflush(stdout);
 	    break;
 	  default:
-	    //if(openSave == SAVE && sel == 1) {
 do_enter_char:
 	    if(sel == 1) 
 	    {
-	      //if((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')  ||
-		//    (ch >= 32 && ch<= 64) || (ch >=123 && ch <= 126)) {	//if it is alphanumeric
 		if(strlen(inputName) >= w-y-9) break;
 		int i;
 		inputName[strlen(inputName)+1] = '\0';
@@ -836,10 +742,8 @@ do_enter_char:
 		inputName[selChar++] = (CAPS || SHIFT) ? ch[0]-32 : ch[0];
 		fprintf(stdout, "\e[%d;%dH", h-2, y+7);
 		setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-		//setScreenColors(BLUE, BGWHITE);
-		printf("%s%*s", inputName, w-y-strlen(inputName)-9, " ");
+		printf("%s%*s", inputName, (int)(w-y-strlen(inputName)-9), " ");
 		fprintf(stdout, "\e[%d;%dH", h-2, y+selChar+7);
-	      //}
 	    }
 	    break;
 	}//end switch
@@ -850,7 +754,6 @@ do_enter_char:
       {//if #1
 	if(!_openFile()) 
 	{
-	  //msgBox("Failed to open file.", OK, ERROR);
 	  return 0;
 	} 
 	FILE_STATE = OPENED;
@@ -896,15 +799,18 @@ int _saveFile()
       if ((lines[i][j] & mask[2]) == mask[2]) fputc(lines[i][j+3], open_file);
     }
     //fputs(lines[i], open_file);
-    if(!LINE_IS_LINKED[i]) fputc('\n', open_file);
+    if(!LINE_IS_LINKED[i] && lines[i][j-4] != '\n') fputc('\n', open_file);
   }
   FILE_STATE = SAVED;
   if(!documentTitle) documentTitle = (char *) malloc(MAX_FILE_NAME_LEN*4);
+  if(!documentTitle) { msgBox("Insufficient memory", OK, ERROR); return 0; }
   if(strcmp(documentTitle, open_file_name) != 0)
+  {
     if(strrchr(open_file_name, '/'))
       strcpy(documentTitle, strrchr(open_file_name, '/')+1);
     else
       strcpy(documentTitle, open_file_name);
+  }
   fflush(open_file); 
 
   checkFileExtension();
@@ -924,12 +830,12 @@ int _openFile()
   //make a backup copy of the opened file, so if things went
   //wrong, we don't lose our original file!!
   char *backup_file_name = (char *) malloc(strlen(open_file_name)+2);
+  if(!backup_file_name) { msgBox("Insufficient memory", OK, ERROR); return 0; }
   strcpy(backup_file_name, open_file_name);
   strcat(backup_file_name, "~\0");
   FILE *backup_file = fopen(backup_file_name, "w");
   
   total_tabs = 0;
-  char buf[(MAX_CHARS_PER_LINE*4)+1];
   int j = 0;
   int i = 0;
   totalLines = 0;
@@ -938,97 +844,97 @@ int _openFile()
   {
     if(!lines[i]) lines[i] = (char *) 
 	  malloc((MAX_CHARS_PER_LINE+TAB_CHARS)*4);
+    if(!lines[i]) { msgBox("Insufficient memory", OK, ERROR); return 0; }
     strcpy(lines[i], "\0");
   }//end for
   i = 0;
-  while(fgets(buf, sizeof(buf), open_file)) 
+  int k = 0;
+  while(!feof(open_file))
   {
-    fputs(buf, backup_file);
     if(totalLines >= MAX_LINES) break;
-      //if this line contains only \n and last line is linked to it,
-      //this means it is the remnant of last line and need not be added
-      if((strlen(buf) == 1) && buf[0] == '\n' && LINE_IS_LINKED[i-1])
-      { LINE_IS_LINKED[i-1] = 0; continue; }
-      int k = 0;
-      if((strlen(buf)+strlen(lines[i])) > MAX_CHARS_PER_LINE) 
-      {
-	k = MAX_CHARS_PER_LINE-strlen(lines[i]);
-	strncat(lines[i], buf, k);
-	//buf += k;
-	LINE_IS_LINKED[i] = 1;
-	totalLines++; i++;
-      }
-      if(buf[strlen(buf)-1] == '\n') 
-      {//if ends in \n, remove it
-	strncat(lines[i], buf+k, strlen(buf)-k-1);
-	LINE_IS_LINKED[i] = 0;
-      } 
-      else 
-      {
-	strncat(lines[i], buf+k, strlen(buf)-k);
-	LINE_IS_LINKED[i] = 1;
-      }
-      totalCharsInLine[i] = strlen(lines[i]);
-      totalLines++; i++;
-    //}
-  }//end while
-  //fclose(open_file);
+    j = fgetc(open_file);
+    fputc(j, backup_file);
+    if(k == MAX_CHARS_PER_LINE)
+    {
+      LINE_IS_LINKED[i] = 1;
+      i++; k = 0;
+      totalLines++;
+    }
+    lines[i][k] = j;
+    if(j == '\n')
+    {
+      LINE_IS_LINKED[i] = 0;
+      i++; k = 0;
+      totalLines++;
+    }
+    else
+    {
+      k++;
+    }
+  }
   fflush(backup_file);
   
   char *t = (char *)malloc(MAX_CHARS_PER_LINE*4);
+  if(!t) { msgBox("Insufficient memory", OK, ERROR); return 0; }
   for(i = 0; i < totalLines; i++) 
   {
     /////////////////////////////////////////////////////////
     //we will expand all chars in line into 4-byte length..
     /////////////////////////////////////////////////////////
     int x, z;
+    char c;
+    int carry = 0;
+    int len;
+    len = strlen(lines[i]);
     for(x = 0; x < strlen(lines[i]); x++) 
     {
+      if(x+carry >= MAX_CHARS_PER_LINE*4)
+      {
+	if(!LINE_IS_LINKED[i])
+	{
+	  move_lines_down(totalLines, i+1);
+	  lines[i+1][0] = '\0';
+	  LINE_IS_LINKED[i] = 1;
+	}
+	strcpy(t, lines[i]+x);
+	lines[i][x] = '\0';
+	strcpy(tmp2, lines[i+1]);
+	strcpy(lines[i+1], t);
+	strcat(lines[i+1], tmp2);
+      }
+
       z = 1;
       if ((lines[i][x] & mask[0]) == mask[0]) z++;
       if ((lines[i][x] & mask[1]) == mask[1]) z++;
       if ((lines[i][x] & mask[2]) == mask[2]) z++;
       strcpy(t, lines[i]+x+z);
+      c = lines[i][x];
+      if(c == '\0')
+      {
+	break;
+      }
       int w;
       for(w = z; w < 4; w++) lines[i][x+w] = ' ';
       lines[i][x+w] = '\0';
       strcat(lines[i], t);
+      
+      if(c == '\t')
+      {
+	j = TAB_CHARS -(((x/4)+carry)%TAB_CHARS);
+	j = 0 ? TAB_CHARS : j;
+	carry += (j-1)*4;
+	len += carry;
+      }
       x += (w-1);
     }
-    ///////////////////////////////////////////////////
-    
-    if(strlen(lines[i])/4 > MAX_CHARS_PER_LINE) 
-    {
-      if(LINE_IS_LINKED[i])
-      {
-	char *tmp = (char *) malloc(MAX_CHARS_PER_LINE*4);
-	strcpy(tmp, lines[i+1]);
-	strcpy(lines[i+1], lines[i]+(MAX_CHARS_PER_LINE*4));
-	lines[i][MAX_CHARS_PER_LINE*4] = '\0';
-	strcat(lines[i+1], tmp);
-	free(tmp);
-      } 
-      else 
-      {
-	for(j = totalLines; j > i+1; j--) 
-	{
-	  strcpy(lines[j], lines[j-1]);
-	  totalCharsInLine[j] = strlen(lines[j])/4;
-	  LINE_IS_LINKED[j] = LINE_IS_LINKED[j-1];
-	}
-	strcpy(lines[i+1], lines[i]+(MAX_CHARS_PER_LINE*4));
-	lines[i][MAX_CHARS_PER_LINE*4] = '\0';
-	totalLines++;
-      }
-      //lines[i][MAX_CHARS_PER_LINE] = '\0';
-      LINE_IS_LINKED[i] = 1;
-      //shiftAllTabs(i+2, 1);
-    }
-    totalCharsInLine[i] = strlen(lines[i])/4;
+    //int len2;
+    //len2 = strlen(lines[i]);
+    calcTotalCharsInLine(i);
   }
   free(t);
   
   if(!documentTitle) documentTitle = (char *) malloc(MAX_FILE_NAME_LEN*4);
+  if(!documentTitle) { msgBox("Insufficient memory", OK, ERROR); return 0; }
   if(strrchr(open_file_name, '/'))
     strcpy(documentTitle, strrchr(open_file_name, '/')+1);
   else
