@@ -1,6 +1,6 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2013, 2014 (c)
+ *    Copyright 2013, 2014, 2015 (c)
  * 
  *    file: defs.h
  *    This file is part of Prime.
@@ -18,6 +18,7 @@
  *    You should have received a copy of the GNU General Public License
  *    along with Prime.  If not, see <http://www.gnu.org/licenses/>.
  */    
+
 /**********************************************
   This file contains definitions for constants 
   and functions used by the Prime program.
@@ -28,6 +29,7 @@
 #define __Prime_H
 
 #include "../corelib/dialogs.h"
+#include <errno.h>
 
 #define MAXDIRS		255
 #define	MAXFILES	255
@@ -41,6 +43,8 @@ int MAX_MSG_BOX_H;
 void drawMenuBar(int x, int y, int w);
 void init();
 void scanDir(char *dir);
+void refreshAll();
+void refreshWindows();
 void refreshDirView();
 void refreshFileView();
 void refreshBottomView();
@@ -48,9 +52,12 @@ void showFileMenu(int visible);
 void showEditMenu(int visible);
 void showOptionsMenu(int visible);
 void showHelpMenu(int visible);
+void showPropertiesDialog();
+char *file_open_location();
 void toggleSelected();
 void showReadMe();	//called from help menu to show README file
 void showKeybindings();	//called from help menu to show KEYBINDINGS file
+void exit_gracefully();
 
 //Menu items//
 typedef struct menu_item_s 
@@ -58,6 +65,7 @@ typedef struct menu_item_s
   char *mtext;
   char *shortCutKeys;
 } menu_item_struct;
+
 //Main menu items//
 typedef struct menu_s 
 {
@@ -72,11 +80,11 @@ char *menu[4];
 #define totalMainMenus	4	//total items in main menu bar
 #define fTotal		5	//total items in file menu
 #define eTotal		7	//total items in edit menu
-#define oTotal		2	//total items in options menu
+#define oTotal		3	//total items in options menu
 #define hTotal		4	//total items in help menu
 char *fileMenu[6];
 char *editMenu[8];
-char *optionsMenu[2];
+char *optionsMenu[3];
 char *helpMenu[5];
 
 char *dirs[MAXDIRS];
@@ -112,12 +120,13 @@ void exportTreeFromCommandLine(char *d, char *f);
 void scanThisDir(char tmp[], int level, int showProgress);
 
 char *cwd;	//the current working directory
-char *log_file_name;//string holding the name of the log file
-FILE *log_file;
+/* defined in args.c */
+extern char *log_file_name;//string holding the name of the log file
+extern FILE *log_file;
 
-char *config_file_name;//string holding the name of the config file
-FILE *config_file;
-int GNU_DOS_LEVEL;
+extern char *config_file_name;//string holding the name of the config file
+extern FILE *config_file;
+extern int GNU_DOS_LEVEL;
 
 //Array to store different colors that will be given
 //to files of different types.. The array is indexed by
@@ -131,4 +140,7 @@ int GNU_DOS_LEVEL;
 //'h' for hidden files
 //'p' for picture files
 char FILE_DIR_COLOR[26];
+
+/* defined in args.c */
+extern char *prime_ver;
 #endif
\ No newline at end of file
