@@ -1,6 +1,6 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2013, 2014 (c)
+ *    Copyright 2013, 2014, 2015 (c)
  * 
  *    file: edit.c
  *    This file is part of Prime.
@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <dirent.h>
 #include <sys/stat.h>
+#include <errno.h>
 #include <unistd.h>
 
 int CONFIRM_CUT_COPY;
@@ -39,6 +40,21 @@ static int one(const struct dirent *unused)
 
 struct stat statbuf;
 
+mode_t get_mode(__mode_t st_mode)
+{
+  mode_t mode = 00;
+  if(st_mode & S_IRUSR) { mode += 0400; }
+  if(st_mode & S_IWUSR) { mode += 0200; }
+  if(st_mode & S_IXUSR) { mode += 0100; }
+  if(st_mode & S_IRGRP) { mode +=  040; }
+  if(st_mode & S_IWGRP) { mode +=  020; }
+  if(st_mode & S_IXGRP) { mode +=  010; }
+  if(st_mode & S_IROTH) { mode +=   04; }
+  if(st_mode & S_IWOTH) { mode +=   02; }
+  if(st_mode & S_IXOTH) { mode +=   01; }
+  return mode;
+}
+
 void clearSelection() 
 {
   int a = activeWindow;
@@ -85,7 +101,6 @@ void markAll()
     }	//end for
     refreshFileView();
   }	//end if
-  //numStarred += i-1;
   numStarred += j;
   refreshBottomView();
 }
@@ -122,7 +137,6 @@ void unMarkAll()
     }
     refreshFileView();
   }
-  //numStarred -= i+1;
   numStarred -= j;
   refreshBottomView();
 }
@@ -317,9 +331,6 @@ void pasteMarked()
   for(i = 0; i < numCopyFiles; i++) 
   {
     if(ABORT_COPY) goto finish;
-    //c = getKey();
-    //if(c == CTRL_X_KEY || c == CTRL_C_KEY  || c == CTRL_Z_KEY 
-    //   || c == CTRL_Q_KEY) goto finish;
     copyThisFile(copyFiles[i]);
   } //END FOR
   numCopyFiles = 0;
@@ -330,9 +341,6 @@ void pasteMarked()
   for(i = 0; i < numCutFiles; i++) 
   {
     if(ABORT_COPY) goto finish;
-    //c = getKey();
-    //if(c == CTRL_X_KEY || c == CTRL_C_KEY  || c == CTRL_Z_KEY 
-    //   || c == CTRL_Q_KEY) goto finish;
     copyThisFile(cutFiles[i]);
     if(remove(cutFiles[i])) 
     {	//success returns 0, otherwise err value
@@ -348,9 +356,6 @@ void pasteMarked()
   for(i = 0; i < numCopyDirs; i++) 
   {
     if(ABORT_COPY) goto finish;
-    //c = getKey();
-    //if(c == CTRL_X_KEY || c == CTRL_C_KEY  || c == CTRL_Z_KEY 
-    //   || c == CTRL_Q_KEY) goto finish;
     copyThisDir(copyDirs[i], 0);
   }  
   numCopyDirs = 0;
@@ -361,9 +366,6 @@ void pasteMarked()
   for(i = 0; i < numCutDirs; i++) 
   {
     if(ABORT_COPY) goto finish;
-    //c = getKey();
-    //if(c == CTRL_X_KEY || c == CTRL_C_KEY  || c == CTRL_Z_KEY 
-    //   || c == CTRL_Q_KEY) goto finish;
     moveThisDir(cutDirs[i], 0);
   }  
   numCutDirs = 0;
@@ -384,33 +386,35 @@ finish:
  * **************************************/
 void copyThisFile(char *fileName) 
 {
-  char *str;// = (char *) malloc(strlen(fileName));
+  char *str;
   FILE *inFile;
   FILE *outFile;
   char buf[4096];
   int read_size;
 
-  if(!(inFile = fopen(fileName, "r"))) 
+  if(!(inFile = fopen(fileName, "rb"))) 
   {//check the file for reading
-      fprintf(log_file, "Error opening file for read: %s\n", fileName);
+	fprintf(log_file, "Error opening file for read: %s\n", fileName);
 	char *s = (char *) malloc(strlen(fileName)+36);
+	if(!s) { msgBox("Insufficient memory", OK, ERROR); return; }
 	strcpy(s, "Error opening read file:\n");
 	strcat(s, fileName);
 	strcat(s, "\nAborting.");
 	strcat(s, "\0");
 	msgBox(s, OK, ERROR); 
-	fprintf(log_file, "Error opening file '%s'..\n", fileName);
+	//fprintf(log_file, "Error opening file '%s'..\n", fileName);
 	free(s); return;
   } //end if
   //find the last '/' in the path, add one for the start of the filename
   str = strrchr(fileName, '/'); str++; 
-  if((outFile = fopen(str, "r"))) 
+  if((outFile = fopen(str, "rb"))) 
   { //check if out file exists
       int j;
       fclose(outFile);
       if(!CONFIRM_CUT_COPY) 
       {
 	char *s = (char *) malloc(strlen(fileName)+34);
+	if(!s) { msgBox("Insufficient memory", OK, ERROR); return; }
 	strcpy(s, "File:\n");
 	strcat(s, fileName);
 	strcat(s, "\nalready exists. Overwrite?");
@@ -423,18 +427,18 @@ void copyThisFile(char *fileName)
 	if(j == ALL) CONFIRM_CUT_COPY = 1;
       }
   } //end if
-  if(!(outFile = fopen(str, "w"))) 
+  if(!(outFile = fopen(str, "wb"))) 
   { //open out file for writing
       fprintf(log_file, "Error opening file for write: %s\n", str);
 	char *s = (char *) malloc(strlen(fileName)+37);
+	if(!s) { msgBox("Insufficient memory", OK, ERROR); return; }
 	strcpy(s, "Error opening write file:\n");
 	strcat(s, fileName);
 	strcat(s, "\nAborting.");
 	strcat(s, "\0");
 	msgBox(s, OK, ERROR); 
-	fprintf(log_file, "Error opening file '%s'..\n", fileName);
+	//fprintf(log_file, "Error opening file '%s'..\n", fileName);
 	free(s); return;
-      //msgBox("Error opening write file!. Aborting.", OK, ERROR); return;
   } //end if
     
   //start copying infile to outfile in chunks//
@@ -445,7 +449,16 @@ void copyThisFile(char *fileName)
     
   fclose(inFile);
   fclose(outFile);
-  //free(--str);
+  /* Set the file mode */
+  struct stat statbuf;
+  mode_t mode = 00;
+  int z = lstat(fileName, &statbuf);
+  if(z == -1) { msgBox(strerror(errno), OK, ERROR); return; }
+  mode = get_mode(statbuf.st_mode);
+  if(chmod(str, mode) != 0) 
+  {
+    msgBox(strerror(errno), OK, ERROR);
+  }
 }
 
 /**********************************************
@@ -454,12 +467,10 @@ void copyThisFile(char *fileName)
  * ********************************************/
 void copyThisDir(char tmp[], int level) 
 {
-  char *str;// = (char *) malloc(strlen(tmp));
+  char *str;
 
-  //int nf = 0;
   int n;
   struct dirent **eps;
-  //struct stat st2;
 
   //find the last '/' in the path, add one for the start of the dirname
   str = strrchr(tmp, '/'); str++; 
@@ -470,7 +481,8 @@ void copyThisDir(char tmp[], int level)
   if(strlen(str) > 58)
     for(n = 0; n < 58; n++) putchar(str[n]);
   else printf("%s", str);
-
+  fflush(stdout);
+  
   //check if it doesn't exist, create it
   if(stat(str, &statbuf) == -1) mkdir(str, 0700);
   else 
@@ -478,6 +490,7 @@ void copyThisDir(char tmp[], int level)
     if(!CONFIRM_CUT_COPY) 
     {
       char *s = (char *) malloc(strlen(str)+39);
+      if(!s) { msgBox("Insufficient memory", OK, ERROR); return; }
       strcpy(s, "Directory:\n");
       strcat(s, str);
       strcat(s, "\nalready exists. Overwrite?");
@@ -490,7 +503,8 @@ void copyThisDir(char tmp[], int level)
       if(n == ALL) CONFIRM_CUT_COPY = 1;
     }
   }
-  chdir(str);
+  int z = chdir(str);
+  if(z == -1) { msgBox("Error changing directory", OK, ERROR); return; }
   
   n = scandir(tmp, &eps, one, alphasort);
   if(n >= 0) 
@@ -502,21 +516,19 @@ void copyThisDir(char tmp[], int level)
 	 || strcmp(eps[cnt]->d_name, "..") == 0)	//ignore "." & ".."
 	continue;
       char *cc = (char *) malloc(strlen(tmp)+strlen(eps[cnt]->d_name)+2);
+      if(!cc) { msgBox("Insufficient memory", OK, ERROR); return; }
       strcpy(cc, tmp);
       strcat(cc, "/");
       strcat(cc, eps[cnt]->d_name);
       strcat(cc, "\0");
       lstat(cc,&statbuf);
-      //lstat(eps[cnt]->d_name,&statbuf);
       if(S_ISDIR(statbuf.st_mode)) 
       {
 	copyThisDir(cc, level+1);
-	//fprintf(log_file, "line 303 copyThisDir(): tmp2:%s\n", cc);
       } 
       else 
       {
 	copyThisFile(cc);
-	//fprintf(log_file, "line 311 copyThisDir(): tmp3:%s\n", cc);
       } //end else
       free(cc);
     } //end for
@@ -526,9 +538,18 @@ void copyThisDir(char tmp[], int level)
     fprintf(log_file, "Error opening dir: %s\n", tmp);
   } //end else
   
-  //} //end outer if
-  chdir("..");
-  //free(--str);
+  /* Set the Directory mode */
+  struct stat statbuf;
+  mode_t mode = 00;
+  z = lstat(tmp, &statbuf);
+  if(z == -1) { msgBox(strerror(errno), OK, ERROR); return; }
+  mode = get_mode(statbuf.st_mode);
+  z = chdir("..");
+  if(z == -1) { msgBox("Error changing directory", OK, ERROR); return; }
+  if(chmod(str, mode) != 0) 
+  {
+	    msgBox(strerror(errno), OK, ERROR);
+  }
 }
 
 /**********************************************
@@ -537,9 +558,8 @@ void copyThisDir(char tmp[], int level)
  * ********************************************/
 void moveThisDir(char tmp[], int level) 
 {
-  char *str;// = (char *) malloc(strlen(tmp));
+  char *str;
 
-  //int nf = 0;
   int n;
   struct dirent **eps;
   struct stat st;
@@ -553,6 +573,7 @@ void moveThisDir(char tmp[], int level)
   if(strlen(str) > 58)
     for(n = 0; n < 58; n++) putchar(str[n]);
   else printf("%s", str);
+  fflush(stdout);
   
   //check if it doesn't exist, create it
   if(stat(str, &st) == -1) mkdir(str, 0700);
@@ -561,6 +582,7 @@ void moveThisDir(char tmp[], int level)
     if(!CONFIRM_CUT_COPY) 
     {
       char *s = (char *) malloc(strlen(str)+39);
+      if(!s) { msgBox("Insufficient memory", OK, ERROR); return; }
       strcpy(s, "Directory:\n");
       strcat(s, str);
       strcat(s, "\nalready exists. Overwrite?");
@@ -573,7 +595,8 @@ void moveThisDir(char tmp[], int level)
       if(n == ALL) CONFIRM_CUT_COPY = 1;
     }
   }
-  chdir(str);
+  int z = chdir(str);
+  if(z == -1) { msgBox("Error changing directory", OK, ERROR); return; }
   
   n = scandir(tmp, &eps, one, alphasort);
   if(n >= 0) 
@@ -585,12 +608,12 @@ void moveThisDir(char tmp[], int level)
 	 || strcmp(eps[cnt]->d_name, "..") == 0)	//ignore "." & ".."
 	continue;
       char *cc = (char *) malloc(strlen(tmp)+strlen(eps[cnt]->d_name)+2);
+      if(!cc) { msgBox("Insufficient memory", OK, ERROR); return; }
       strcpy(cc, tmp);
       strcat(cc, "/");
       strcat(cc, eps[cnt]->d_name);
       strcat(cc, "\0");
       lstat(cc,&statbuf);
-      //lstat(eps[cnt]->d_name,&st);
       if(S_ISDIR(statbuf.st_mode)) 
       {
 	moveThisDir(cc, level+1);
@@ -608,8 +631,19 @@ void moveThisDir(char tmp[], int level)
     fprintf(log_file, "Error opening dir: %s\n", tmp);
   } //end else
   
-  //} //end outer if
-  chdir("..");
+  /* Set the Directory mode */
+  struct stat statbuf;
+  mode_t mode = 00;
+  z = lstat(tmp, &statbuf);
+  if(z == -1) { msgBox(strerror(errno), OK, ERROR); return; }
+  mode = get_mode(statbuf.st_mode);
+  if(chmod(str, mode) != 0) 
+  {
+	    msgBox(strerror(errno), OK, ERROR);
+  }
+
+  z = chdir("..");
+  if(z == -1) { msgBox("Error changing directory", OK, ERROR); return; }
   rmdir(tmp);
 }
 
@@ -633,6 +667,7 @@ void deleteThisDir(char tmp[], int level)
   if(strlen(str) > 58)
     for(n = 0; n < 58; n++) putchar(str[n]);
   else printf("%s", str);
+  fflush(stdout);
   
   //chdir(tmp);
   n = scandir(str, &eps, one, alphasort);
@@ -645,6 +680,7 @@ void deleteThisDir(char tmp[], int level)
 	 || strcmp(eps[cnt]->d_name, "..") == 0)	//ignore "." & ".."
 	continue;
       char *cc = (char *) malloc(strlen(str)+strlen(eps[cnt]->d_name)+2);
+      if(!cc) { msgBox("Insufficient memory", OK, ERROR); return; }
       strcpy(cc, str);
       strcat(cc, "/");
       strcat(cc, eps[cnt]->d_name);
@@ -652,9 +688,12 @@ void deleteThisDir(char tmp[], int level)
       lstat(cc,&statbuf);
       if(S_ISDIR(statbuf.st_mode)) 
       {
-	chdir(str);
+	int z;
+	z = chdir(str);
+	if(z == -1) { msgBox("Error changing directory", OK, ERROR); return; }
 	deleteThisDir(eps[cnt]->d_name, level+1);
-	chdir("..");
+	z = chdir("..");
+	if(z == -1) { msgBox("Error changing directory", OK, ERROR); return; }
       } 
       else 
       {
@@ -668,22 +707,14 @@ void deleteThisDir(char tmp[], int level)
     fprintf(log_file, "Error opening dir: %s\n", tmp);
   } //end else
   
-  //chdir("..");
   rmdir(str);
   free(str);
 }
 
 void deleteThisFile(char *fileName) 
 {
-	if(strcmp(fileName, "(Empty folder)") == 0) 
-	  return;//this means folder is empty!!
-	//char *s = (char *) malloc(strlen(fileName)+33);
-	//strcpy(s, "Are you sure you want to delete\n");
-	//strcat(s, fileName);
-	//strcat(s, "\0");
-	//int k = msgBox(s, YES|NO, CONFIRM);
-	//free(s);
-	//if(k == YES) remove(fileName);
+  if(strcmp(fileName, "(Empty folder)") == 0)
+    return;//this means folder is empty!!
 }
 
 /********************************************
@@ -701,7 +732,7 @@ void deleteMarked()
     if(i == NO) return;
     for(i = 0; i < totalFiles; i++) 
     {
-      if(fileStar[i] == '*') 
+      if(fileStar[i] == '*')
       {
 	remove(files[i]);
       }//end if
