@@ -1,6 +1,6 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2013, 2014 (c)
+ *    Copyright 2013, 2014, 2015 (c)
  * 
  *    file: find.c
  *    This file is part of Prime.
@@ -24,7 +24,7 @@
 #include <unistd.h>
 #include <pwd.h>
 
-extern int getKey();
+extern char *tmp, *tmp2;
 FILE *searchResultsFile;
 
 static int one(const struct dirent *unused) 
@@ -32,6 +32,27 @@ static int one(const struct dirent *unused)
   return 1;
 }
 
+void draw_findFile_buttons(int bx, int by, int selection)
+{
+  if(selection == 4)
+    setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_HBUTTONS]);
+  else setScreenColors(FG_COLOR[COLOR_BUTTONS], BG_COLOR[COLOR_BUTTONS]);
+  fprintf(stdout, "\x1b[%d;%dH", bx, by);
+  fprintf(stdout, "   OK   ");
+  by += 12;
+  if(selection == 5)
+    setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_HBUTTONS]);
+  else setScreenColors(FG_COLOR[COLOR_BUTTONS], BG_COLOR[COLOR_BUTTONS]);
+  fprintf(stdout, "\x1b[%d;%dH", bx, by);
+  fprintf(stdout, " CANCEL ");
+  //adjust cursor to point at selected button
+  if(selection == 4)
+    fprintf(stdout, "\x1b[%d;%dH", bx, by-9);
+  else if(selection == 5)
+    fprintf(stdout, "\x1b[%d;%dH", bx, by+1);
+  fflush(stdout);
+}
+
 /*************************************************
  * this function displays a dialog box for the
  * user to enter file name, where to search, and
@@ -42,9 +63,10 @@ static int one(const struct dirent *unused)
 void findFile() 
 {
   int x = 5;
-  int y = 5;
-  int w = 45;
+  int w = 46;
   int h = 12+x;
+  int y = (SCREEN_W-w)/2;
+  w += y;
   int i, sel = 0;	//the selected control in the window
   //sel: 	0=first input field	1=first option
   //		2=second option		3=third option
@@ -57,11 +79,11 @@ void findFile()
   input1_len = 0; input2_len = 0;
   highlight1 = 0; highlight2 = 0;
   
-  if(!(searchResultsFile = fopen("/tmp/find", "w+"))) 
+  if(!(searchResultsFile = fopen("/tmp/prime_find", "w+"))) 
   {
     msgBox("Error opening temporary file. Aborting.", OK, ERROR);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-    fprintf(log_file, "Failed to open /tmp/find file..\n");
+    fprintf(log_file, "Failed to open '/tmp/prime_find' file..\n");
     return;
   }
 
@@ -72,9 +94,9 @@ void findFile()
   fprintf(stdout, "Enter file name to search for:");
   setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
   fprintf(stdout, "\e[%d;%dH", x+3, y+2);
-  for(i = 0; i < MAX_INPUT1_LEN; i++) putchar(' ');
+  printf("%*s", MAX_INPUT1_LEN, " ");
   fprintf(stdout, "\e[%d;%dH", x+9, y+22);
-  for(i = 0; i < MAX_INPUT2_LEN; i++) putchar(' ');
+  printf("%*s", MAX_INPUT2_LEN, " ");
   setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
   fprintf(stdout, "\e[%d;%dH", x+6, y+2);
   fprintf(stdout, "Search in:");
@@ -85,20 +107,16 @@ void findFile()
   fprintf(stdout, "\e[%d;%dH", x+9, y+2);
   fprintf(stdout, "[ ] Under this path:");
   //Draw the buttons//
+  showCursor();
   int bx, by;
-    bx = h-1;
-    by = y + ((w-16)/2) - 2;
-    setScreenColors(FG_COLOR[COLOR_BUTTONS], BG_COLOR[COLOR_BUTTONS]);
-    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-    fprintf(stdout, "   OK   ");
-    by += 12;
-    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-    fprintf(stdout, " CANCEL ");
-    fprintf(stdout, "\e[%d;%dH", x+3, y+2);//adjust cursor to be at input field
+  bx = h-1;
+  by = y + ((w-y-16)/2) - 2;
+  draw_findFile_buttons(bx, by, sel);
+  fprintf(stdout, "\e[%d;%dH", x+3, y+2);//adjust cursor to be at input field
 
-   //wait for user response//
+  //wait for user response//
  int ch;
- while(1) 
+ while(1)
  {	//infinite program loop//
     ch = getKey();
     switch(ch) 
@@ -178,55 +196,46 @@ void findFile()
 	  if(osel == 0) 
 	  {	//search in current working directory
 	    char *t = (char *) malloc(strlen(cwd));//[strlen(cwd)];
+	    if(!t) { msgBox("Insufficient memory", OK, ERROR); return; }
+	    
 	    fprintf(log_file, "Searching for '%s' under '%s'..\n", 
 		    findFileName, cwd);
 	    strcpy(t, cwd);
-	    //chdir("..");
-	    //drawBox(5, 5, SCREEN_H, SCREEN_W-5, " Search results: ", YES);
 	    scanDirForFile(t, 0);
-	    //chdir(cwd);
 	    free(t);
-	    //fclose(searchResultsFile);
-	    remove("/tmp/find");
+	    remove("/tmp/prime_find");
 	    return;
 	  } 
 	  else if (osel == 1) 
 	  {	//search starting from '/'
-	    //drawBox(5, 5, SCREEN_H, SCREEN_W-5, " Search results: ", YES);
 	    fprintf(log_file, "Searching for '%s' under Root..\n", findFileName);
 	    scanDirForFile("/", 0);
+	    return;
 	  } 
 	  else if (osel == 2) 
 	  {	//search in selected path
 	    if(input2_len == 0)
-	      msgBox("You should enter a valid path.", OK, ERROR);
+	      msgBox("Invalid path.", OK, ERROR);
 	    else 
 	    {
-	      //check for '~'
-	      char *tmp = (char *) malloc(MAX_DIR_NAME_LEN);
 	      strcpy(tmp, findInDir);
 	      if(strchr(tmp, '~')) 
 	      {
-		char *tmp2 = (char *) malloc(MAX_DIR_NAME_LEN);
 		strcpy(tmp2, findInDir+((strchr(tmp, '~')+1)-tmp));
 		struct passwd *pass;//will be used to find the home dir
 		if((pass = getpwuid(geteuid())))
-		  ;//printf("\nError: couldn't open home directory.\n");
+		  ;
 		strcpy(tmp, pass->pw_dir);
 		strcat(tmp, "/");
 		strcat(tmp, tmp2);
 		strcat(tmp, "\0");
-		free(tmp2);
 	      }
-	      //scanDirForFile(findInDir, 0);
 	      fprintf(log_file, "Searching for '%s' under '%s'..\n", 
 		      findFileName, tmp);
 	      scanDirForFile(tmp, 0);
-	      free(tmp);
 	    }
 	  }
-	  //fclose(searchResultsFile);
-	  remove("/tmp/find");
+	  remove("/tmp/prime_find");
 	  return;				//otherwise return the input
 	}
 	if(sel == 6) return;		//return NULL also if selected CANCEL
@@ -235,6 +244,7 @@ void findFile()
 	if(sel == 0) 
 	{	//first input field
 	  if(highlight1 >= input1_len) break;//already at last char
+	  if(highlight1 == MAX_INPUT1_LEN-1) break;//already at last char
 	  highlight1++;
 	  //adjust cursor to point at input field
 	  fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
@@ -242,20 +252,21 @@ void findFile()
 	else if(sel == 4) 
 	{	//second input field
 	  if(highlight2 >= input2_len) break;//already at last char
+	  if(highlight2 == MAX_INPUT2_LEN-1) break;//already at last char
 	  highlight2++;
 	  //adjust cursor to point at input field
 	  fprintf(stdout, "\x1b[%d;%dH", x+9, y+highlight2+22);
 	} 
 	break;
       case(LEFT_KEY):
-	if(sel == 0) 
+	if(sel == 0)
 	{	//first input field
 	  if(highlight1 <= 0) break;//already at first char
 	  highlight1--;
 	  //adjust cursor to point at input field
 	  fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
 	} 
-	else if(sel == 4) 
+	else if(sel == 4)
 	{	//second input field
 	  if(highlight2 <= 0) break;//already at first char
 	  highlight2--;
@@ -265,7 +276,7 @@ void findFile()
 	break;
       case(TAB_KEY):
 	  bx = h-1;
-	  by = y + ((w-16)/2) - 2;
+	  by = y + ((w-y-16)/2) - 2;
 	  if(sel == 0) 
 	  {
 	    fprintf(stdout, "\e[%d;%dH", x+7, y+3);
@@ -287,39 +298,18 @@ void findFile()
 	    sel = 4;
 	  } 
 	  else if(sel == 4) 
-	  {	    
-	    setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_HBUTTONS]);
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-	    fprintf(stdout, "   OK   ");
-	    by += 12;
-	    setScreenColors(FG_COLOR[COLOR_BUTTONS], BG_COLOR[COLOR_BUTTONS]);
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-	    fprintf(stdout, " CANCEL ");
+	  {
+	    draw_findFile_buttons(bx, by, sel);
 	    sel = 5;
-	    //adjust cursor to point at "CANCEL"
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by-9);
 	  } 
 	  else if(sel == 5) 
 	  {
-	    setScreenColors(FG_COLOR[COLOR_BUTTONS], BG_COLOR[COLOR_BUTTONS]);
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-	    fprintf(stdout, "   OK   ");
-	    by += 12;
-	    setScreenColors(FG_COLOR[COLOR_HBUTTONS], BG_COLOR[COLOR_HBUTTONS]);
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-	    fprintf(stdout, " CANCEL ");
+	    draw_findFile_buttons(bx, by, sel);
 	    sel = 6;
-	    //adjust cursor to point at "OK"
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by+1);
 	  } 
 	  else 
 	  {
-	    setScreenColors(FG_COLOR[COLOR_BUTTONS], BG_COLOR[COLOR_BUTTONS]);
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-	    fprintf(stdout, "   OK   ");
-	    by += 12;
-	    fprintf(stdout, "\x1b[%d;%dH", bx, by);
-	    fprintf(stdout, " CANCEL ");
+	    draw_findFile_buttons(bx, by, sel);
 	    sel = 0;
 	    //adjust cursor to point at input field
 	    fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
@@ -327,7 +317,7 @@ void findFile()
 	setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
 	break;
       case(BACKSPACE_KEY):
-	if(sel == 0) 
+	if(sel == 0)
 	{
 	  if(input1_len == 0) break;
 	  if(highlight1 == 0) break;
@@ -344,7 +334,7 @@ void findFile()
 	  //adjust cursor to point at input field
 	  fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
 	} 
-	else if(sel == 4) 
+	else if(sel == 4)
 	{
 	  if(input2_len == 0) break;
 	  if(highlight2 == 0) break;
@@ -361,6 +351,41 @@ void findFile()
 	  //adjust cursor to point at input field
 	  fprintf(stdout, "\x1b[%d;%dH", x+9, y+highlight2+22);
 	}
+      case(DEL_KEY):
+	if(sel == 0)
+	{
+	  if(input1_len == 0) break;
+	  if(highlight1 == 0) break;
+	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], 
+			  BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
+	  for(i = highlight1; i < input1_len-1; i++) 
+	    findFileName[i] = findFileName[i+1];
+	  findFileName[i] = '\0';
+	  input1_len--;
+	  //adjust cursor to point at input field
+	  fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
+	  for(i = highlight1; i < input1_len; i++) putchar(findFileName[i]);
+	  putchar(' ');
+	  //adjust cursor to point at input field
+	  fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
+	} 
+	else if(sel == 4)
+	{
+	  if(input2_len == 0) break;
+	  if(highlight2 == 0) break;
+	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], 
+			  BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
+	  for(i = highlight2; i < input2_len-1; i++) 
+	    findInDir[i] = findInDir[i+1];
+	  findInDir[i] = '\0';
+	  input2_len--;
+	  //adjust cursor to point at input field
+	  fprintf(stdout, "\x1b[%d;%dH", x+9, y+highlight2+22);
+	  for(i = highlight2; i < input2_len; i++) putchar(findInDir[i]);
+	  putchar(' ');
+	  //adjust cursor to point at input field
+	  fprintf(stdout, "\x1b[%d;%dH", x+9, y+highlight2+22);
+	}
       default:
 	  if((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z')
 	     || (ch >= 32 && ch<= 64) || (ch >=123 && ch <= 126)) 
@@ -371,9 +396,9 @@ enterNewChar:
 	      if(strlen(findFileName) >= MAX_INPUT1_LEN) break;
 	      setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], 
 			  BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	    //inserting in the middle of a string means we need to shift all
-	    //chars one position to the right before inserting the new char
-	    //at the highlighted position.
+	      //inserting in the middle of a string means we need to shift all
+	      //chars one position to the right before inserting the new char
+	      //at the highlighted position.
 	      if(findFileName[highlight1] != '\0') 
 	      {
 		for(i = input1_len; i > highlight1; i--) 
@@ -385,15 +410,24 @@ enterNewChar:
 	      if(input1_len > highlight1) 
 	      {	//there are some chars to the right side
 		//adjust cursor to point at input field
-		//fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlightChar+1);
 		for(i = highlight1; i < input1_len; i++) 
 		  putchar(findFileName[i]);
 	      }
+	      if(highlight1 >= MAX_INPUT1_LEN) highlight1 = MAX_INPUT1_LEN-1;
 	      //adjust cursor to point at input field
 	      fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlight1+2);
 	    } 
 	    else if(sel == 4) 
 	    {	//the second input field
+	      if(osel != 2)
+	      {
+		setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+		fprintf(stdout, "\e[%d;%dH ", x+7+osel, y+3);
+		fprintf(stdout, "\e[%d;%dHX", x+9, y+3);
+		fprintf(stdout, "\e[%d;%dH", x+9, y+3);
+		osel = 2;
+		fprintf(stdout, "\x1b[%d;%dH", x+9, y+highlight2+22);
+	      }
 	      if(strlen(findInDir) >= MAX_INPUT2_LEN) break;
 	      setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], 
 			  BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
@@ -411,17 +445,15 @@ enterNewChar:
 	      if(input2_len > highlight2) 
 	      {	//there are some chars to the right side
 		//adjust cursor to point at input field
-		//fprintf(stdout, "\x1b[%d;%dH", x+3, y+highlightChar+1);
 		for(i = highlight2; i < input2_len; i++) putchar(findInDir[i]);
 	      }
+	      if(highlight2 >= MAX_INPUT2_LEN) highlight2 = MAX_INPUT2_LEN-1;
 	      //adjust cursor to point at input field
 	      fprintf(stdout, "\x1b[%d;%dH", x+9, y+highlight2+22);
 	    }
 	  }
     }
  }
-  //do {;} while (!getchar());
-  //free(tmpC);
   setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
   return;
 }
@@ -439,10 +471,9 @@ enterNewChar:
 void scanDirForFile(char tmp[], int level) 
 {
   static int nf;
-  int n, i;
+  int n, i, z;
   struct dirent **eps;
 
-  //fprintf(log_file, "now in line 41 of scanThisDir(), tmp=%s\n", tmp);
   if(level == 0) 
   {
     nf = 0;
@@ -452,20 +483,23 @@ void scanDirForFile(char tmp[], int level)
     printf("Scanning:");
   }
   
-  chdir(tmp);
+  z = chdir(tmp);
+  if(z == -1)
+  {
+    /* msgBox("Error changing directory", OK, ERROR); */ return;
+  }
   n = scandir(tmp, &eps, one, alphasort);
-  //fprintf(log_file, "now in line 83 of scanThisDir(), n=%d\n", n);
   if(n >= 0) 
   {
-    int cnt;//, c;
-    fprintf(log_file, "directory of: %s, children:%d\n", tmp, n);
+    int cnt;
+    //fprintf(log_file, "directory of: %s, children:%d\n", tmp, n);
     for(cnt = 0; cnt < n; ++cnt) 
     {
-//      c = getKey();
-//      if(c == CTRL_X_KEY || c == CTRL_C_KEY  || c == CTRL_Z_KEY 
-//	  || c == CTRL_Q_KEY) goto stop_scan;
-      
-      lstat(eps[cnt]->d_name,&statbuf);
+      z = lstat(eps[cnt]->d_name,&statbuf);
+      if(z == -1)
+      {
+	/* msgBox(strerror(errno), OK, ERROR); */ return;
+      }
       if(S_ISDIR(statbuf.st_mode)) 
       {
 	if(strcmp(eps[cnt]->d_name, ".") == 0 
@@ -473,56 +507,47 @@ void scanDirForFile(char tmp[], int level)
 	  continue;
 
 	//display progress message to user
-	//fprintf(stdout, "\e[%d;%dH", SCREEN_H/2, 6);
-	//for(i = 0; i < SCREEN_W-11; i++) printf(" ");
 	fprintf(stdout, "\e[%d;%dH", SCREEN_H/2, 6);
 	if(strlen(eps[cnt]->d_name) > SCREEN_W-11)
 	  printf("..%s", 
 		 eps[cnt]->d_name+(strlen(eps[cnt]->d_name)-SCREEN_W-13));
 	else 
 	  printf("%s%*s", 
-		 eps[cnt]->d_name, SCREEN_W-11-strlen(eps[cnt]->d_name), " ");
+		 eps[cnt]->d_name, (int)(SCREEN_W-12-strlen(eps[cnt]->d_name)), " ");
 
 	
 	char *tmp2 = (char *)malloc(strlen(tmp)+strlen(eps[cnt]->d_name)+2);
+	if(!tmp2) { msgBox("Insufficient memory", OK, ERROR); return; }
 	strcpy(tmp2, tmp);
-	strcat(tmp2, "/");
+	if(tmp2[strlen(tmp2)-1] != '/') strcat(tmp2, "/");
 	strcat(tmp2, eps[cnt]->d_name);
 	strcat(tmp2, "\0");
 	
-	//chdir(tmp2);
-	//scanDirForFile(eps[cnt]->d_name, level+1);
 	scanDirForFile(tmp2, level+1);
-	//chdir("..");
 	free(tmp2);
 	
 	continue;
       } 
       else 
       {
-	//if((i = (int)strstr(eps[cnt]->d_name, findFileName))) {
 	if(strrchr(eps[cnt]->d_name, '/'))
 	  i = compareFileName(strrchr(eps[cnt]->d_name, '/')+1);
 	else i = compareFileName(eps[cnt]->d_name);
-	//if((i = compareFileName(eps[cnt]->d_name))) {
 	if(i) 
 	{
 	  //make sure the match is in the filename, not the dir name
 	  //do this by comparing the index of the match with the
 	  //last '/' in the name.
-	  if(i < (int)strrchr(eps[cnt]->d_name, '/')) continue;
+	  if(i < (int)(strrchr(eps[cnt]->d_name, '/')-eps[cnt]->d_name)) continue;
 	  //if the file name is too long, truncate it
 	  if((strlen(tmp)+strlen(eps[cnt]->d_name)) > SCREEN_W-19) 
 	  {
-	    //int j = strlen(tmp)+strlen(eps[cnt]->d_name)-20;
 	    int j = SCREEN_W-strlen(eps[cnt]->d_name)-22;
 	    for(i = 0; i < j; i++)
 	      fputc(tmp[i], searchResultsFile);
 	    fputs("../", searchResultsFile);
 	    fputs(eps[cnt]->d_name, searchResultsFile);
 	    fputc('\n', searchResultsFile);
-	    //fprintf(searchResultsFile, "%*s../%s\n",strlen(tmp)-16,
-		//    tmp, eps[cnt]->d_name);
 	  } 
 	  else 
 	  {
@@ -530,27 +555,25 @@ void scanDirForFile(char tmp[], int level)
 	  }//end inner if
 	  nf++;
 	}//end outer if
-	//fprintf(log_file, "file: %s, cnt:%d\n", eps[cnt]->d_name, cnt);
       }//end if
     }//end for
   }
   else 
   {
-    fprintf(log_file, "Error opening dir: %s\n", tmp);
+    //fprintf(log_file, "Error opening dir: %s\n", tmp);
   }//end outer if
 
-//stop_scan:
-
+  printf("%s\n", tmp);
   if(level == 0) 
   {
     if(nf == 0) 
     {
       msgBox("Search failed. No matches were found.", OK, INFO);
-      fprintf(log_file, "Search failed. No matches were found.\n");
+      //fprintf(log_file, "Search failed. No matches were found.\n");
     } 
     else 
     {
-      fprintf(log_file, "Search finished. %d files were found\n", nf);
+      //fprintf(log_file, "Search finished. %d files were found\n", nf);
       fprintf(searchResultsFile, "%06d", nf);
       fflush(searchResultsFile);
       showSearchResults();
@@ -558,39 +581,48 @@ void scanDirForFile(char tmp[], int level)
     }
   setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
   } 
-  else chdir("..");
-  
-  //scanDir(cwd);
+  else
+  {
+    z = chdir("..");
+    if(z == -1)
+    {
+      /* msgBox("Error changing directory", OK, ERROR); */
+      return;
+    }
+  }
 }
 
 int compareFileName(char *name) 
 {
     int i = 0;
-    //int j = (strstr(name, endStr)-name);
-    //j += strlen(endStr);
-    //int k = strlen(name);
     
     if(strlen(startStr))
       if(strcmp(startStr, "*") != 0)
+      {
 	if(((strstr(name, startStr))-name) == 0) 
 	{ 
 	  i = 1;
 	} 
 	else return 0;
+      }
     if(strlen(endStr))
       if(strcmp(endStr, "*") != 0) 
+      {
 	if(((strstr(name, endStr)-name)+strlen(endStr) == strlen(name))) 
 	{ 
 	  i = 1;
 	} 
 	else return 0;
+      }
     if(strlen(middleStr))
       if(strcmp(middleStr, "*") != 0)
+      {
 	if(((strstr(name, middleStr))-name) > 0) 
 	{ 
 	  i = 1;
 	} 
 	else i = 0;
+      }
     return i;
 }
 
@@ -632,28 +664,37 @@ void showSearchResults()
   j = 6; //counter to keep track of the total number of lines
   k = 6; //counter of char number in the line
   l = 1; //counter to tell the number of the file over the total no.
-  //fprintf(stdout, "\e[%d;%dH", j, k);
   lineStart[0] = 0;
-  fprintf(stdout, "\e[%d;%dH(%d/%d) ", j, k, l, totalResults);
+  int len = 0;
+  char t[20];
+  fprintf(stdout, "\e[%d;%dH", j, k);
+  sprintf(t, "(%d/%d) ", l, totalResults);
+  fprintf(stdout, "%s", t);
+  len = strlen(t);
   while(i < buf_len) 
   {
     if(j > SCREEN_H-6) { moreLines = 1; break; }
-    if(k > SCREEN_W-14)
-    { 
+    if(len > SCREEN_W-12)
+    {
       k = 6; j++; lineStart[l] = i; 
       fprintf(stdout, "\e[%d;%dH   ", j, k);
+      len = 3;
       continue; 
     }
     if(buf[i] == '\n') 
-    { 
+    {
       k = 6; j++; lineStart[l] = ++i; l++;
       if(l > totalResults) break;
       if(j > SCREEN_H-6) { moreLines = 1; break; }
-      fprintf(stdout, "\e[%d;%dH(%d/%d) ", j, k, l, totalResults);
-      continue; 
+      sprintf(t, "(%d/%d) ", l, totalResults);
+      fprintf(stdout, "\e[%d;%dH", j, k);
+      fprintf(stdout, "%s", t);
+      len = strlen(t);
+      continue;
     }
     putchar(buf[i++]);
     k++;
+    len++;
   }//end while
   char c;
   while((c = getKey())) 
@@ -662,6 +703,7 @@ void showSearchResults()
     {
       case(SPACE_KEY):
       case(ENTER_KEY):
+      case(ESC_KEY):
 	return;
 	break;
       case(UP_KEY):
@@ -671,14 +713,11 @@ void showSearchResults()
 	{
 	  //reset to beginning of next page
 	  fseek(searchResultsFile, 
-		//buf_size*page_number
 		page_start[page_number-1], 
 		SEEK_SET);
 	  buf_len = fread(buf, sizeof(char), sizeof(buf), searchResultsFile);
 	  if(!buf_len) return;
 	  buf_len -= 6;
-	  //page_start[page_number] = 
-	  //    page_start[page_number-1]+lineStart[firstVisLine+1];
 	  firstPageEntry = page_firstVisFile[page_number];
 	  firstVisLine = page_firstVisFile[page_number]-1;
 	  page_number--;
@@ -687,10 +726,9 @@ void showSearchResults()
 	  while(i < buf_len)
 	  {
 	    if(buf[i] == '\n')
-	    { lineStart[++j] == i; }
+	    { lineStart[++j] = i; }
 	    i++;
 	  }
-	  //lineStart[0] = page_start[page_number];
 	  goto Refresh;
 	}
 	break;
@@ -701,7 +739,6 @@ void showSearchResults()
 	{
 	  //reset to beginning of next page
 	  fseek(searchResultsFile, 
-		//buf_size*page_number
 		page_start[page_number]+lineStart[firstVisLine+1], 
 		SEEK_SET);
 	  buf_len = fread(buf, sizeof(char), sizeof(buf), searchResultsFile);
@@ -719,28 +756,35 @@ void showSearchResults()
 	break;
 Refresh:
     i = lineStart[firstVisLine]; j = 6; k = 6; moreLines=0; l=firstVisLine+1;
-    //fprintf(stdout,"\e[%d;%dH(%d/%s) ",j,k, firstVisLine+j-5, totalResults);
-    fprintf(stdout, "\e[%d;%dH(%d/%d) ", j, k, l+page_firstVisFile[page_number], totalResults);
+    fprintf(stdout, "\e[%d;%dH", j, k);
+    sprintf(t, "(%d/%d) ", l+page_firstVisFile[page_number], totalResults);
+    fprintf(stdout, "%s", t);
+    len = strlen(t);
     while(i < buf_len) 
     {
       if(j > SCREEN_H-6) { moreLines = 1; break; }
-      if(k > SCREEN_W-14) 
+      if(len > SCREEN_W-12)
       { 
 	k = 6; j++; lineStart[l] = i; 
 	fprintf(stdout, "\e[%d;%dH   ", j, k);
+	len = 3;
 	continue; 
       }
       if(buf[i] == '\n') 
       { 
-	printf("%*s", SCREEN_W-k-19, " ");//finish line with spaces
+	printf("%*s", SCREEN_W-len-12, " ");//finish line with spaces
 	k = 6; j++; lineStart[l] = ++i; l++;
 	if(l+page_firstVisFile[page_number] > totalResults) break;
 	if(j > SCREEN_H-6) { moreLines = 1; break; }
-	fprintf(stdout, "\e[%d;%dH(%d/%d) ", j, k, l+page_firstVisFile[page_number], totalResults);
-	continue; 
+	fprintf(stdout, "\e[%d;%dH", j, k);
+	sprintf(t, "(%d/%d) ", l+page_firstVisFile[page_number], totalResults);
+	fprintf(stdout, "%s", t);
+	len = strlen(t);
+	continue;
       }
       putchar(buf[i++]);
       k++;
+      len++;
      }//end while
     }//end switch
 
