@@ -1,6 +1,6 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2013, 2014 (c)
+ *    Copyright 2013, 2014, 2015 (c)
  * 
  *    file: main.c
  *    This file is part of Prime.
@@ -39,8 +39,14 @@
 #include <pwd.h>
 #include <time.h>
 
+extern void showPropertiesDialog();
+
+char *tmp, *tmp2;
+char *input_str;
+
 struct passwd *pass;//will be used to find the home dir
 time_t tm;
+static sig_atomic_t end = 0;
 
 static int one(const struct dirent *unused);
 
@@ -55,21 +61,26 @@ int NEW_GNU_DOS_LEVEL = 0;
 
 extern void parse_args(int argc, char **argv);
 
-extern void showFileMenu(int visible);
-extern void showHelpMenu(int visible);
-extern void showEditMenu(int visible);
-//extern int msgBox(char *msg, int Buttons, msgType tmsg);
+void exit_gracefully()
+{
+  fprintf(log_file, "Restoring terminal [ OK ]..\n");
+  restoreTerminal(); 
+  fprintf(log_file, "Closing files and Exiting Prime [ OK ]..\n");
+  setScreenColors(WHITE, BGDEFAULT);
+  clearScreen(); 
+  showCursor();
+  fflush(stdout);
+  time_t tm = time(NULL);
+  fprintf(log_file, "System time and date: %s", ctime(&tm));
+  write_config_file();
+  fcloseall();
+  //freePrinters();
+  setScreenColors(WHITE, BGDEFAULT);
+  clearScreen(); 
+  fprintf(stdout, "\x1b[0m");	/* reset settings */
+  exit(0);
+}
 
-//extern void setScreenColors(int FG, int BG);
-//extern void getScreenSize();
-//extern void clearScreen();
-
-extern void cutMarked();
-extern void cutOne();
-extern void markAll();
-extern void unMarkAll();
-
-//static sig_atomic_t end = 0;
 /***************************************
  * sighandler(): 
  * Procedure that sets trap to the system
@@ -90,24 +101,11 @@ void sighandler(int signo)
       int i = msgBox("Are you sure you want to exit?", YES | NO, CONFIRM);
       if(i == YES) 
       { 	//exit gracefully
-	clearScreen(); 
-	setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	fprintf(log_file, "Restoring terminal [ OK ]..\n");
-	restoreTerminal();
-	tm = time(NULL);
-	fprintf(log_file, "Closing files and Exiting Prime [ OK ]..\n");
-	fprintf(log_file, "System time and date: %s", ctime(&tm));
-	write_config_file();
-	fcloseall();
-	//freePrinters();
-	clearScreen(); 
-	exit(0); 
+	exit_gracefully();
       } 
-      else 
+      else
       {
-	setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	refreshFileView();
-	refreshDirView();
+	refreshWindows();
       }//end inner if
     } 
     else 
@@ -119,30 +117,71 @@ void sighandler(int signo)
 struct termios oldtio, curtio;
 struct sigaction sa;
 
+void print_dir_highlight(int selectedDir)
+{
+  int j;
+  fprintf(stdout, "\x1b[%d;%dH%s", selectedDir+4, 3, dirHighLight);
+  if(strlen(dirs[firstVisDir+selectedDir]) >= MAX_DIR_NAME_LEN)
+  {
+    fprintf(stdout, "\x1b[%d;%dH%c", selectedDir+4, 3,
+		dirStar[firstVisDir+selectedDir]);
+    for(j = 0; j < MAX_DIR_NAME_LEN-3; j++)
+      fprintf(stdout, "%c", dirs[firstVisDir+selectedDir][j]);
+    fprintf(stdout, "..");
+  }
+  else
+  {
+    fprintf(stdout, "\x1b[%d;%dH%c%s", selectedDir+4, 3, 
+		dirStar[firstVisDir+selectedDir], 
+		dirs[firstVisDir+selectedDir]);
+  }//end if
+}
+
+void print_file_highlight(int selectedFile)
+{
+  int j;
+  fprintf(stdout, "\x1b[%d;%dH%s", selectedFile+4, (SCREEN_W/2)+1, fileHighLight);
+  if(strlen(files[firstVisFile+selectedFile]) >= MAX_DIR_NAME_LEN) 
+  {
+    fprintf(stdout, "\x1b[%d;%dH%c", selectedFile+4, (SCREEN_W/2)+1, 
+	    fileStar[firstVisFile+selectedFile]);
+    for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
+      fprintf(stdout, "%c", files[firstVisFile+selectedFile][j]);
+    fprintf(stdout, "..");
+  } 
+  else 
+  { 
+    fprintf(stdout, "\x1b[%d;%dH%c%s", selectedFile+4, (SCREEN_W/2)+1, 
+	    fileStar[firstVisFile+selectedFile], 
+	    files[firstVisFile+selectedFile]);
+  }//end if
+}
+
 /***************************************
  * main(): 
  * Main program loop.
  * *************************************/
 int main(int argc, char **argv) 
 {
-  parse_args(argc, argv);
+ parse_args(argc, argv);
  
  cwd = getcwd(NULL, 0);
  tm = time(NULL);
  if(!log_file)
  {
-	    printf("Couldn't open logfile. If you want to start Prime with no "
-		   "log, type '%s --no-log'\n", argv[0]);
-	    exit(1);
+   printf("Couldn't open logfile. If you want to start Prime with no "
+	  "log, type '%s --no-log'\n", argv[0]);
+   exit(1);
  }
  fprintf(log_file, "Starting Prime in: %s\n", cwd);
  fprintf(log_file, "System time and date: %s", ctime(&tm));
  fprintf(log_file, "Log_file: %s\n", log_file_name);
  fprintf(log_file, "Program started: in the main function..\n");
- init();			//some initialization code
  fprintf(log_file, "Initializing variables [ OK ]..\n");
- read_config_file();
+ init();			//some initialization code
+ 
  fprintf(log_file, "Reading configuration file [ OK ]..\n");
+ read_config_file();
  
  clearScreen();			//clear the screen
  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
@@ -220,44 +259,17 @@ do_up:
        } 
        else 
        {
-	int j;
+	//int j;
 	setScreenColors(FILE_DIR_COLOR[dirType[firstVisDir+selectedDir+1]-'a'], BG_COLOR[COLOR_WINDOW]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedDir+5, 3, dirHighLight);
-	if(strlen(dirs[firstVisDir+selectedDir+1]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedDir+5, 3, 
-		  dirStar[firstVisDir+selectedDir+1]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", dirs[firstVisDir+selectedDir+1][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedDir+5, 3, 
-		    dirStar[firstVisDir+selectedDir+1], 
-		    dirs[firstVisDir+selectedDir+1]);
-	}//end if
+	print_dir_highlight(selectedDir+1);
 	setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedDir+4, 3, dirHighLight);
-	if(strlen(dirs[firstVisDir+selectedDir]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedDir+4, 3, 
-		  dirStar[firstVisDir+selectedDir]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", dirs[firstVisDir+selectedDir][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ fprintf(stdout, "\x1b[%d;%dH%c%s", selectedDir+4, 3, 
-		    dirStar[firstVisDir+selectedDir], 
-		    dirs[firstVisDir+selectedDir]);
-	}//end if
+	print_dir_highlight(selectedDir);
        }//end if
        fprintf(stdout, "\e[3;3H");	//move cursor to point to window
      } 
      else 
      {
-       int j;
+       //int j;
        if(strcmp(files[0], "(Empty folder)") == 0) break;//this dir is empty!!
        selectedFile--;
        if(selectedFile < 0) 
@@ -268,39 +280,9 @@ do_up:
        else 
        {
 	setScreenColors(FILE_DIR_COLOR[fileType[firstVisFile+selectedFile+1]-'a'], BG_COLOR[COLOR_WINDOW]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedFile+5, 
-		(SCREEN_W/2)+1, fileHighLight);
-	if(strlen(files[firstVisFile+selectedFile+1]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedFile+5, (SCREEN_W/2)+1, 
-		  fileStar[firstVisFile+selectedFile+1]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", files[firstVisFile+selectedFile+1][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedFile+5, (SCREEN_W/2)+1, 
-		    fileStar[firstVisFile+selectedFile+1], 
-		    files[firstVisFile+selectedFile+1]);
-	}//end if
+	print_file_highlight(selectedFile+1);
 	setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedFile+4, 
-		(SCREEN_W/2)+1, fileHighLight);
-	if(strlen(files[firstVisFile+selectedFile]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedFile+4, (SCREEN_W/2)+1, 
-		  fileStar[firstVisFile+selectedFile]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", files[firstVisFile+selectedFile][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedFile+4, (SCREEN_W/2)+1, 
-		    fileStar[firstVisFile+selectedFile], 
-		    files[firstVisFile+selectedFile]);
-	}//end if
+	print_file_highlight(selectedFile);
        }//end if
        fprintf(stdout, "\e[3;%dH",(SCREEN_W/2)+2);//cursor to point to window
      } 
@@ -323,39 +305,11 @@ do_down:
        } 
        else 
        {
-	 int j;
+	//int j;
 	setScreenColors(FILE_DIR_COLOR[dirType[firstVisDir+selectedDir-1]-'a'], BG_COLOR[COLOR_WINDOW]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedDir+3, 3, dirHighLight);
-	if(strlen(dirs[firstVisDir+selectedDir-1]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedDir+3, 3, 
-		  dirStar[firstVisDir+selectedDir-1]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", dirs[firstVisDir+selectedDir-1][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedDir+3, 3, 
-		    dirStar[firstVisDir+selectedDir-1], 
-		    dirs[firstVisDir+selectedDir-1]);
-	}//end if
+	print_dir_highlight(selectedDir-1);
 	setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedDir+4, 3, dirHighLight);
-	if(strlen(dirs[firstVisDir+selectedDir]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedDir+4, 3, 
-		  dirStar[firstVisDir+selectedDir]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", dirs[firstVisDir+selectedDir][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedDir+4, 3, 
-		    dirStar[firstVisDir+selectedDir], 
-		    dirs[firstVisDir+selectedDir]);
-	}//end if
+	print_dir_highlight(selectedDir);
        }//end if
        fprintf(stdout, "\e[3;3H");	//move cursor to point to window
      } 
@@ -374,41 +328,11 @@ do_down:
        } 
        else 
        {
-	 int j;
+	//int j;
 	setScreenColors(FILE_DIR_COLOR[fileType[firstVisFile+selectedFile-1]-'a'], BG_COLOR[COLOR_WINDOW]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedFile+3, 
-		(SCREEN_W/2)+1, fileHighLight);
-	if(strlen(files[firstVisFile+selectedFile-1]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedFile+3, 
-		  (SCREEN_W/2)+1, fileStar[firstVisFile+selectedFile-1]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", files[firstVisFile+selectedFile-1][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedFile+3, (SCREEN_W/2)+1, 
-		    fileStar[firstVisFile+selectedFile-1], 
-	   files[firstVisFile+selectedFile-1]);
-	}//end if
+	print_file_highlight(selectedFile-1);
 	setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	fprintf(stdout, "\x1b[%d;%dH%s", selectedFile+4, 
-		(SCREEN_W/2)+1, fileHighLight);
-	if(strlen(files[firstVisFile+selectedFile]) >= MAX_DIR_NAME_LEN) 
-	{
-	  fprintf(stdout, "\x1b[%d;%dH%c", selectedFile+4, 
-		  (SCREEN_W/2)+1, fileStar[firstVisFile+selectedFile]);
-	for(j = 0; j < MAX_DIR_NAME_LEN-3; j++) 
-	  fprintf(stdout, "%c", files[firstVisFile+selectedFile][j]);
-	fprintf(stdout, "..");
-	} 
-	else 
-	{ 
-	  fprintf(stdout, "\x1b[%d;%dH%c%s", selectedFile+4, (SCREEN_W/2)+1, 
-		    fileStar[firstVisFile+selectedFile], 
-		    files[firstVisFile+selectedFile]);
-	}//end if
+	print_file_highlight(selectedFile);
        }//end if
        fprintf(stdout, "\e[3;%dH",(SCREEN_W/2)+2);//cursor to point to window
      } 
@@ -434,7 +358,15 @@ do_down:
       if(activeWindow == DIR_WIN) 
       {	//navigate to the selected directory
         scanDir(dirs[firstVisDir+selectedDir]);
-      } break;
+      }
+      else
+      {
+       if(strcmp(files[firstVisFile+selectedFile], "(Empty folder)") == 0)  break;
+       showPropertiesDialog();
+       setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+       scanDir(cwd);
+      }
+      break;
      }
    case(SPACE_KEY):		//toggle select/unselect file or directory
      toggleSelected();
@@ -444,6 +376,7 @@ do_down:
      {
       if(GNU_DOS_LEVEL <= 4) break;
       findFile();
+      hideCursor();
       setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
       scanDir(cwd);
      } 
@@ -453,14 +386,13 @@ do_down:
      if(ALT) 
      {
       //if(GNU_DOS_LEVEL > 1) break;
-      fprintf(log_file, "Opening file menu..\n");
+      //fprintf(log_file, "Opening file menu..\n");
       showFileMenu(YES);
-      fprintf(log_file, "Closed file menu, Waiting for user input..\n");
+      //fprintf(log_file, "Closed file menu, Waiting for user input..\n");
       break;
      } 
      else if(CTRL) 
      {
-   //case(CTRL_F_KEY):			//open the find file dialog box
       if(GNU_DOS_LEVEL > 4) break;
       findFile();
       setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
@@ -468,38 +400,32 @@ do_down:
      } 
      else goto insert_char;
      break;
-   //case(ALT_E_KEY):			//showup the edit menu
    case('e'):
      if(CTRL) 
      {
        if(GNU_DOS_LEVEL > 2) goto do_end;
-   //case(CTRL_E_KEY):
       exportTree(YES);
       setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
       scanDir(cwd);
      }
      else if(ALT) 
      {
-      //if(GNU_DOS_LEVEL > 1) break;
-      fprintf(log_file, "Opening edit menu..\n");
+      //fprintf(log_file, "Opening edit menu..\n");
       showEditMenu(YES);
-      fprintf(log_file, "Closed edit menu, Waiting for user input..\n");
+      //fprintf(log_file, "Closed edit menu, Waiting for user input..\n");
       break;
      } 
      else goto insert_char;
      break;
-   //case(ALT_H_KEY):			//showup the help menu
    case('h'):
      if(ALT) 
      {
-      //if(GNU_DOS_LEVEL > 1) break;
-      fprintf(log_file, "Opening help menu..\n");
+      //fprintf(log_file, "Opening help menu..\n");
       showHelpMenu(YES);
-      fprintf(log_file, "Closed help menu, Waiting for user input..\n");
+      //fprintf(log_file, "Closed help menu, Waiting for user input..\n");
      } 
      else goto insert_char;
      break;
-   //case(CTRL_X_KEY):			//CTRL-X --> cut starred items
    case('x'):
      if(CTRL) 
      {
@@ -519,8 +445,7 @@ do_down:
 	  else if(ch == 'f' && CTRL) goto do_open_location;
 	  else if(ch == 'g' && CTRL)
 	  {
-	    refreshDirView();
-	    refreshFileView();
+	    refreshWindows();
 	    loop = 0;
 	    break;
 	  }
@@ -530,7 +455,6 @@ do_down:
      } 
      else goto insert_char;
      break;
-   //case(CTRL_V_KEY):			//call paste marked function
    case('v'):
      if(CTRL) 
      {
@@ -539,7 +463,6 @@ do_down:
      } 
      else goto insert_char;
      break;
-   //case(CTRL_P_KEY):			//call paste marked function
    case('p'):
      if(CTRL) 
      {
@@ -547,13 +470,10 @@ do_down:
       msgBox("Oops! This function is currently not implemented.", OK, INFO);
       //fprintf(log_file, "Showing Print dialog box..\n");
       //showPrintDialogBox();
-      setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	refreshFileView();
-	refreshDirView();
+      refreshWindows();
      }
      else goto insert_char;
      break;
-   //case(CTRL_A_KEY):			//mark all
    case('a'):
      if(CTRL) 
      {
@@ -562,14 +482,10 @@ do_down:
       if(activeWindow == DIR_WIN)
 	fprintf(log_file, "Selecting All directories..\n");
       else fprintf(log_file, "Selecting All files..\n");
-      setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-      refreshFileView();
-      refreshDirView();
-      refreshBottomView();
+      refreshAll();
      } 
      else goto insert_char;
      break;
-   //case(CTRL_O_KEY):			//open location..
    case('w'):
      if(CTRL) 
      {
@@ -590,40 +506,13 @@ do_down:
     {
      if(GNU_DOS_LEVEL > 4) break;
 do_open_location:
-     if(1) ;
-     char *res = inputBox("Enter directory path to open:", "Open Location");
-     if(res != NULL) 
-     {
-      //************** check for '~'
-      char *tmp = (char *) malloc(MAX_DIR_NAME_LEN);
-      strcpy(tmp, res);
-      if(strchr(tmp, '~')) 
-      {
-	char *tmp2 = (char *) malloc(MAX_DIR_NAME_LEN);
-	strcpy(tmp2, res+((strchr(res, '~')+1)-res));
-	struct passwd *pass;//will be used to find the home dir
-	if((pass = getpwuid(geteuid())))
-	  ;//printf("\nError: couldn't open home directory.\n");
-	strcpy(tmp, pass->pw_dir);
-	strcat(tmp, "/");
-	strcat(tmp, tmp2);
-	strcat(tmp, "\0");
-	free(tmp2);
-      }//***************
-      scanDir(tmp);
-      free(tmp);
-	//scanDir(res);
-	refreshFileView();
-	refreshDirView();
-     }
-     fprintf(log_file, "Opening '%s'..\n", res);
-     //free(res);
+     file_open_location();
     } 
     else if(ALT)
     {//open options menu
-      fprintf(log_file, "Opening options menu..\n");
+      //fprintf(log_file, "Opening options menu..\n");
       showOptionsMenu(YES);
-      fprintf(log_file, "Closed options menu, Waiting for user input..\n");
+      //fprintf(log_file, "Closed options menu, Waiting for user input..\n");
     }
     else goto insert_char;
     break;
@@ -672,7 +561,6 @@ do_end:
        refreshFileView();
      }//end outer else
      break;
-   //case(CTRL_Q_KEY):
    case('g'):
     if(CTRL) 
     {
@@ -697,29 +585,15 @@ do_exit:
       int i = msgBox("Are you sure you want to exit?", YES | NO, CONFIRM);
       if(i == YES) 
       { 	//exit gracefully
-	setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	clearScreen(); 
-	fprintf(log_file, "Restoring terminal [ OK ]..\n");
-	restoreTerminal();
-	tm = time(NULL);
-	fprintf(log_file, "Closing files and Exiting Prime [ OK ]..\n");
-	fprintf(log_file, "System time and date: %s", ctime(&tm));
-	write_config_file();
-	fcloseall();
-	setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	clearScreen(); 
-	exit(0); 
+	exit_gracefully();
       } 
       else 
       {
-	setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	refreshFileView();
-	refreshDirView();
+	refreshWindows();
       }//end inner if
     } 
     else goto insert_char;
     break;
-   //case(CTRL_C_KEY):
    case('c'):
      if(CTRL) 
      {
@@ -727,7 +601,6 @@ do_exit:
      } 
      else goto insert_char;
      break;
-   //case(CTRL_Z_KEY):
    case('/'):
      if(CTRL && GNU_DOS_LEVEL > 4) goto do_undo;
      break;
@@ -743,19 +616,21 @@ do_undo:
      } 
      else goto insert_char;
      break;
-   //case(CTRL_N_KEY):
    case('n'):
    if(CTRL) 
    {
     if(GNU_DOS_LEVEL > 1) goto do_down;
     inputBox("Enter directory name to create:", "New Directory");
     char *res2 = (char *) malloc(MAX_DIR_NAME_LEN);
+    if(!res2) { msgBox("Insufficient memory", OK, ERROR); break; }
+    
     strcpy(res2, input);
-    if(res2 != NULL) {
+    if(res2 != NULL)
+    {
       struct stat st;
       if(stat(res2, &st) == -1) 
       {
-	mkdir(res2, 0700);//check if it doesn't exist, create it
+	mkdir(res2, 0775);//check if it doesn't exist, create it
 	fprintf(log_file, "Creating directory %s..\n", res2);
       }
       else 
@@ -844,16 +719,9 @@ insert_char:
   }//end switch
   setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
  }//end while
- 
- fprintf(log_file, "Restoring terminal [ OK ]..\n");
- restoreTerminal();
- fprintf(log_file, "Closing files and Exiting Prime [ OK ]..\n");
- time_t tm = time(NULL);
- fprintf(log_file, "System time and date: %s", ctime(&tm));
- write_config_file();
- fcloseall();
- //freePrinters();
- exit(0);
+
+ exit_gracefully();
+ return 0;
 }
 
 void toggleSelected() 
@@ -919,6 +787,13 @@ void toggleSelected()
   }
 }
 
+#define SHOW_ERROR_AND_RETURN(err)	\
+({					\
+  msgBox(err, OK, ERROR);		\
+  refreshFileView();			\
+  refreshDirView();			\
+  return;				\
+})
 
 void scanDir(char *dir) 
 {
@@ -929,7 +804,10 @@ void scanDir(char *dir)
   struct stat statbuf;
 
   n = scandir(dir, &eps, one, alphasort);
-  chdir(dir);
+  j = chdir(dir);
+  if(j == -1) SHOW_ERROR_AND_RETURN("Error changing directory");
+  //{ msgBox("Error changing directory", OK, ERROR); return; }
+  
   cwd = getcwd(NULL, 0);
   dcount = fcount = 0;
   
@@ -939,7 +817,10 @@ void scanDir(char *dir)
     fprintf(log_file, "Scanning directory '%s'\n", cwd);
     for(cnt = 0; cnt < n; ++cnt) 
     {
-      lstat(eps[cnt]->d_name,&statbuf);
+      j = lstat(eps[cnt]->d_name,&statbuf);
+      if(j == -1) SHOW_ERROR_AND_RETURN(strerror(errno));
+      //{ msgBox(strerror(errno), OK, ERROR); return; }
+      
       if(S_ISDIR(statbuf.st_mode)) 
       {
 	if(dcount >= MAXDIRS) break;
@@ -971,28 +852,30 @@ void scanDir(char *dir)
 	{
 	  if(files[fcount-1][0] == '.')	//is it hidden?
 	    fileType[fcount-1] = 'h';
-	  else if(strstr(files[fcount-1], ".tar")
-	    || strstr(files[fcount-1], ".gz")
-	    || strstr(files[fcount-1], ".xz")
-	    || strstr(files[fcount-1], ".Z")
-	    || strstr(files[fcount-1], ".rar")
-	    || strstr(files[fcount-1], ".zip")
-	    || strstr(files[fcount-1], ".bz2")
-	    || strstr(files[fcount-1], ".7z")
-	    || strstr(files[fcount-1], ".lzma")
-	    || strstr(files[fcount-1], ".lha")
-	    || strstr(files[fcount-1], ".jar"))//is archive?
+	  else if(strcasestr(files[fcount-1], ".tar")
+	       || strcasestr(files[fcount-1], ".gz")
+	       || strcasestr(files[fcount-1], ".xz")
+	       || strcasestr(files[fcount-1], ".Z")
+	       || strcasestr(files[fcount-1], ".rar")
+	       || strcasestr(files[fcount-1], ".zip")
+	       || strcasestr(files[fcount-1], ".bz2")
+	       || strcasestr(files[fcount-1], ".7z")
+	       || strcasestr(files[fcount-1], ".lzma")
+	       || strcasestr(files[fcount-1], ".lha")
+	       || strcasestr(files[fcount-1], ".jar"))//is archive?
 	    fileType[fcount-1] = 'a';
-	  else if(strstr(files[fcount-1], ".bmp")
-	    || strstr(files[fcount-1], ".png")
-	    || strstr(files[fcount-1], ".jpg")
-	    || strstr(files[fcount-1], ".jpeg")
-	    || strstr(files[fcount-1], ".gif")
-	    || strstr(files[fcount-1], ".tiff"))//is picture?
+	  else if(strcasestr(files[fcount-1], ".bmp")
+	       || strcasestr(files[fcount-1], ".png")
+	       || strcasestr(files[fcount-1], ".jpg")
+	       || strcasestr(files[fcount-1], ".jpeg")
+	       || strcasestr(files[fcount-1], ".pcx")
+	       || strcasestr(files[fcount-1], ".ico")
+	       || strcasestr(files[fcount-1], ".gif")
+	       || strcasestr(files[fcount-1], ".tiff"))//is picture?
 	    fileType[fcount-1] = 'p';
 	  else if((statbuf.st_mode & S_IXUSR)
-	    || (statbuf.st_mode & S_IXGRP)
-	    || (statbuf.st_mode & S_IXOTH)) //is executable?
+	       || (statbuf.st_mode & S_IXGRP)
+	       || (statbuf.st_mode & S_IXOTH)) //is executable?
 	    fileType[fcount-1] = 'x';
 	  else	//just a regular file
 	    fileType[fcount-1] = 'r';
@@ -1005,29 +888,25 @@ void scanDir(char *dir)
     fprintf(log_file, "Error opening dir: %s\n", cwd);
     dcount = msgBox("Cannot open directory.\nProbably bad path specified "
 		    "or access denied.", OK, ERROR);
-    setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-    refreshFileView();
-    refreshDirView();
+    refreshWindows();
     return;
   }
-
-    totalFiles = fcount;
-    if(totalFiles < numVisFiles) 
-      files[totalFiles] = NULL;	//so the refreshFileView() will work properly
-    firstVisFile = 0;
-    selectedFile = 0;
+  
+  totalFiles = fcount;
+  if(totalFiles < numVisFiles)
+    files[totalFiles] = NULL;	//so the refreshFileView() will work properly
+  firstVisFile = 0;
+  selectedFile = 0;
     
-    totalDirs = dcount;
-    if(totalDirs < numVisDirs) 
-      dirs[totalDirs] = NULL;	//so the refreshDirView() will work properly
-    firstVisDir = 0;
-    selectedDir = 0;
+  totalDirs = dcount;
+  if(totalDirs < numVisDirs) 
+    dirs[totalDirs] = NULL;	//so the refreshDirView() will work properly
+  firstVisDir = 0;
+  selectedDir = 0;
     
-    numStarred = 0;
-    refreshFileView();
-    refreshDirView();
-    refreshBottomView();
-    fprintf(log_file, "Finished scanning directory '%s'..\n", cwd);
+  numStarred = 0;
+  refreshAll();
+  //fprintf(log_file, "Finished scanning directory '%s'..\n", cwd);
 }
 
 static int one(const struct dirent *unused) 
@@ -1036,6 +915,23 @@ static int one(const struct dirent *unused)
 }
 
 
+void refreshAll()
+{
+  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+  drawBox(1,1, SCREEN_H, SCREEN_W," Prime for GNU/Linux ",NO);
+  drawMenuBar(2, 2, SCREEN_W-2);
+  refreshFileView();
+  refreshDirView();
+  refreshBottomView();
+}
+
+void refreshWindows()
+{
+  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+  refreshFileView();
+  refreshDirView();
+}
+
 /***************************************
  * refreshDirView(): 
  * Procedure to refresh the left window
@@ -1051,10 +947,9 @@ void refreshDirView()
  {
    i = msgBox("No directory entries found!.", OK, ERROR);
    refreshFileView();
-   refreshDirView();//this will make an infinite loop.. solution??
+   refreshDirView();// FIXME: this will make an infinite loop.. solution??
    return;
  }
- //setScreenColors(GREEN, BG_COLOR[COLOR_WINDOW]);
  for(i = 0; i < numVisDirs; i++) 
  {
    if(dirs[firstVisDir+i] == NULL) break; 
@@ -1073,7 +968,6 @@ void refreshDirView()
 	fprintf(stdout, "\x1b[%d;%dH%c%s", i+4, 3, 
 		dirStar[firstVisDir+i], dirs[firstVisDir+i]);
       }//end if
-      //setScreenColors(GREEN, BG_COLOR[COLOR_WINDOW]);
     } 
     else 
     {
@@ -1120,7 +1014,6 @@ void refreshFileView()
    files[1] = NULL;
    totalFiles = 1;
  }
- //setScreenColors(GREEN, BG_COLOR[COLOR_WINDOW]);
  for(i = 0; i < numVisFiles; i++) 
  {
     if(files[firstVisFile+i] == NULL) break; 
@@ -1128,7 +1021,8 @@ void refreshFileView()
     {
       setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
       fprintf(stdout, "\x1b[%d;%dH%s", i+4, (SCREEN_W/2)+1, fileHighLight);
-      if(strlen(files[firstVisFile+i]) >= MAX_FILE_NAME_LEN) {
+      if(strlen(files[firstVisFile+i]) >= MAX_FILE_NAME_LEN)
+      {
 	fprintf(stdout, "\x1b[%d;%dH%c", i+4, (SCREEN_W/2)+1, 
 		fileStar[firstVisFile+i]);
 	for(j = 0; j < MAX_FILE_NAME_LEN-3; j++) 
@@ -1140,7 +1034,6 @@ void refreshFileView()
 	fprintf(stdout, "\x1b[%d;%dH%c%s", i+4, (SCREEN_W/2)+1, 
 		      fileStar[firstVisFile+i], files[firstVisFile+i]);
       }//end if
-      //setScreenColors(GREEN, BG_COLOR[COLOR_WINDOW]);
     } 
     else 
     {
@@ -1213,6 +1106,7 @@ void refreshBottomView()
  * **************************************/
 void drawMenuBar(int x, int y, int w) 
 {
+  printf("\x1b[0m");
   setScreenColors(FG_COLOR[COLOR_MENU_BAR], BG_COLOR[COLOR_MENU_BAR]);
   fprintf(stdout, "\x1b[%d;%dH", x, y);		//reposition the cursor
   int i,j, lastChar=y;
@@ -1227,7 +1121,9 @@ void drawMenuBar(int x, int y, int w)
     {
       if(menu[i][j] == '&') 
       {	//turn on underline feature to print the shortcut key
-	fprintf(stdout, "\x1b[4m%c\x1b[24m", menu[i][j+1]);//then turn it off
+	//fprintf(stdout, "\x1b[4m");
+	fprintf(stdout, "%c", menu[i][j+1]);
+	//fprintf(stdout, "\x1b[24m");//then turn it off
       }
       else
 	fprintf(stdout, "%c", menu[i][j+1]);	//print normal chars (other than the
@@ -1243,9 +1139,14 @@ void drawMenuBar(int x, int y, int w)
  * Procedure containing code that init-
  * ializes various data structures.
  * **************************************/
-void init() 
+void init()
 {
   getScreenSize();
+  if(!SCREEN_H || !SCREEN_W)
+  {
+    printf("Error: Failed to get terminal size\n");
+    exit(1);
+  }
   //define colors that will be used to colorize the view
   FILE_DIR_COLOR['d'-'a'] = FG_COLOR[COLOR_WINDOW];	//color for directories
   FILE_DIR_COLOR['x'-'a'] = GREEN;	//color for exec files
@@ -1275,8 +1176,9 @@ void init()
   editMenu[5] = "Clear sel. ^W";
   editMenu[6] = "Find..     ^F";
   //4. Options Menu//
-  optionsMenu[0] = "Change colors  ";
-  optionsMenu[1] = "Reset config   ";
+  optionsMenu[0] = "Properties     ";
+  optionsMenu[1] = "Change colors  ";
+  optionsMenu[2] = "Reset config   ";
   //5. Help Menu//
   helpMenu[0] = "View README    ";
   helpMenu[1] = "GNU keybindings";
@@ -1291,6 +1193,7 @@ void init()
  totalDirs = 0;
  //reserve enough space in memory for highlight bar
  dirHighLight = malloc(SCREEN_W/2);
+ if(!dirHighLight) goto memory_error;
  memset(dirHighLight, ' ', (SCREEN_W/2)-4);	//fill the bar with spaces
  dirHighLight[(SCREEN_W/2)-4] = '\0';
   //set defaults for File view//
@@ -1299,6 +1202,7 @@ void init()
  selectedFile = -1;
  totalFiles = 0;
  fileHighLight = malloc(SCREEN_W/2);
+ if(!fileHighLight) goto memory_error;
  memset(fileHighLight, ' ', (SCREEN_W/2)-2);
  fileHighLight[(SCREEN_W/2)-2] = '\0';
  //strcat(fileHighLight, "\0");
@@ -1312,6 +1216,7 @@ void init()
  {
    dirStar[i] = ' ';
    dirs[i] = (char *) malloc(MAX_DIR_NAME_LEN);
+   if(!dirs[i]) goto memory_error;
    dirType[i] = 'd';
  }
  for(i = 0; i < MAXFILES; i++) 
@@ -1319,6 +1224,7 @@ void init()
    fileStar[i] = ' ';
    fileType[i] = 'r';
    files[i] = (char *) malloc(MAX_FILE_NAME_LEN);
+   if(!files[i]) goto memory_error;
  }
   
  numStarred = 0;
@@ -1330,14 +1236,12 @@ void init()
  numCopyDirs = 0;
  for(i = 0; i < MAX_CUT_COPY; i++) 
  {
-   //cutFiles[i] = malloc(1);
-   //copyFiles[i] = malloc(1);
-   //cutDirs[i] = malloc(1);
-   //copyDirs[i] = malloc(1);
    cutFiles[i] = malloc(MAX_FILE_NAME_LEN);
    copyFiles[i] = malloc(MAX_FILE_NAME_LEN);
    cutDirs[i] = malloc(MAX_DIR_NAME_LEN);
    copyDirs[i] = malloc(MAX_DIR_NAME_LEN);
+   if(!cutFiles[i] || !copyFiles[i] || !cutDirs[i] || !copyDirs[i])
+     goto memory_error;
  }
 
  //numPrinters = -1;
@@ -1345,48 +1249,63 @@ void init()
  //PRINTING = 0;
 
  startStr = (char *) malloc(MAX_DIR_NAME_LEN); 
- middleStr = (char *) malloc(MAX_DIR_NAME_LEN); 
+ if(!startStr) goto memory_error;
+ middleStr = (char *) malloc(MAX_DIR_NAME_LEN);
+ if(!middleStr) goto memory_error;
  endStr = (char *) malloc(MAX_DIR_NAME_LEN); 
+ if(!endStr) goto memory_error;
 
-  FG_COLOR[COLOR_WINDOW] = 37;
-  FG_COLOR[COLOR_HIGHLIGHT_TEXT] = 34;
-  FG_COLOR[COLOR_MENU_BAR] = 34;
-  FG_COLOR[COLOR_STATUS_BAR] = 34;
-  FG_COLOR[COLOR_BUTTONS] = 37;
-  FG_COLOR[COLOR_HBUTTONS] = 32;
-  BG_COLOR[COLOR_WINDOW] = 40; //44;
-  BG_COLOR[COLOR_HIGHLIGHT_TEXT] = 47;
-  BG_COLOR[COLOR_MENU_BAR] = 47;
-  BG_COLOR[COLOR_STATUS_BAR] = 47;
-  BG_COLOR[COLOR_BUTTONS] = 41;
-  BG_COLOR[COLOR_HBUTTONS] = 41;
-  //initiate color arrays
-  COLOR_STR[0] = "BLACK";
-  COLOR_STR[1] = "RED";
-  COLOR_STR[2] = "GREEN";
-  COLOR_STR[3] = "BROWN";
-  COLOR_STR[4] = "BLUE";
-  COLOR_STR[5] = "MAGENTA";
-  COLOR_STR[6] = "CYAN";
-  COLOR_STR[7] = "WHITE";
-  FG_COLOR_ARRAY[0] = 30;
-  FG_COLOR_ARRAY[1] = 31;
-  FG_COLOR_ARRAY[2] = 32;
-  FG_COLOR_ARRAY[3] = 33;
-  FG_COLOR_ARRAY[4] = 34;
-  FG_COLOR_ARRAY[5] = 35;
-  FG_COLOR_ARRAY[6] = 36;
-  FG_COLOR_ARRAY[7] = 37;
-  BG_COLOR_ARRAY[0] = 40;
-  BG_COLOR_ARRAY[1] = 41;
-  BG_COLOR_ARRAY[2] = 42;
-  BG_COLOR_ARRAY[3] = 43;
-  BG_COLOR_ARRAY[4] = 44;
-  BG_COLOR_ARRAY[5] = 45;
-  BG_COLOR_ARRAY[6] = 46;
-  BG_COLOR_ARRAY[7] = 47;
+ FG_COLOR[COLOR_WINDOW]         = 37;
+ FG_COLOR[COLOR_HIGHLIGHT_TEXT] = 34;
+ FG_COLOR[COLOR_MENU_BAR]       = 34;
+ FG_COLOR[COLOR_STATUS_BAR]     = 34;
+ FG_COLOR[COLOR_BUTTONS]        = 37;
+ FG_COLOR[COLOR_HBUTTONS]       = 32;
+ BG_COLOR[COLOR_WINDOW]         = 49;
+ BG_COLOR[COLOR_HIGHLIGHT_TEXT] = 47;
+ BG_COLOR[COLOR_MENU_BAR]       = 47;
+ BG_COLOR[COLOR_STATUS_BAR]     = 47;
+ BG_COLOR[COLOR_BUTTONS]        = 41;
+ BG_COLOR[COLOR_HBUTTONS]       = 41;
+ //initiate color arrays
+ COLOR_STR[0] = "BLACK";
+ COLOR_STR[1] = "RED";
+ COLOR_STR[2] = "GREEN";
+ COLOR_STR[3] = "BROWN";
+ COLOR_STR[4] = "BLUE";
+ COLOR_STR[5] = "MAGENTA";
+ COLOR_STR[6] = "CYAN";
+ COLOR_STR[7] = "WHITE";
+ FG_COLOR_ARRAY[0] = 30;
+ FG_COLOR_ARRAY[1] = 31;
+ FG_COLOR_ARRAY[2] = 32;
+ FG_COLOR_ARRAY[3] = 33;
+ FG_COLOR_ARRAY[4] = 34;
+ FG_COLOR_ARRAY[5] = 35;
+ FG_COLOR_ARRAY[6] = 36;
+ FG_COLOR_ARRAY[7] = 37;
+ BG_COLOR_ARRAY[0] = 49;
+ BG_COLOR_ARRAY[1] = 41;
+ BG_COLOR_ARRAY[2] = 42;
+ BG_COLOR_ARRAY[3] = 43;
+ BG_COLOR_ARRAY[4] = 44;
+ BG_COLOR_ARRAY[5] = 45;
+ BG_COLOR_ARRAY[6] = 46;
+ BG_COLOR_ARRAY[7] = 47;
+ 
+ tmp = (char *) malloc(MAX_DIR_NAME_LEN+MAX_FILE_NAME_LEN+1);
+ if(!tmp) goto memory_error;
+ tmp2 = (char *) malloc(MAX_DIR_NAME_LEN+MAX_FILE_NAME_LEN+1);
+ if(!tmp2) goto memory_error;
+ input_str = (char *) malloc(MAX_FILE_NAME_LEN);
+ if(!input_str) goto memory_error;
 
  initTerminal();
+ return;
+ 
+memory_error:
+ fprintf(stderr, "Insufficient memory\n");
+ exit(1);
 }
 
 
@@ -1401,6 +1320,7 @@ int read_config_file()
 	exit(1);
       }
       config_file_name = (char *) malloc(strlen(pass->pw_dir)+12);
+      if(!config_file_name) { printf("Insufficient memory\n"); exit(1); }
       strcpy(config_file_name, pass->pw_dir);
       strcat(config_file_name, "/");
       strcat(config_file_name, ".prime.conf");
@@ -1488,6 +1408,7 @@ int write_config_file_defaults()
 	return 0;
       }
       config_file_name = (char *) malloc(strlen(pass->pw_dir)+12);
+      if(!config_file_name) { printf("Insufficient memory\n"); exit(1); }
       strcpy(config_file_name, pass->pw_dir);
       strcat(config_file_name, "/");
       strcat(config_file_name, ".prime.conf");
@@ -1509,7 +1430,7 @@ int write_config_file_defaults()
       fprintf(config_file, "FG_COLOR_SBAR = 34\n");
       fprintf(config_file, "FG_COLOR_HBUT = 32\n");
       fprintf(config_file, "FG_COLOR_BUT = 37\n");
-      fprintf(config_file, "BG_COLOR_WIN = 40\n");
+      fprintf(config_file, "BG_COLOR_WIN = 49\n");
       fprintf(config_file, "BG_COLOR_HLT = 47\n");
       fprintf(config_file, "BG_COLOR_MBAR = 47\n");
       fprintf(config_file, "BG_COLOR_SBAR = 47\n");
