@@ -1,6 +1,6 @@
 /* 
  *    Programmed By: Mohammed Isam Mohammed [mohammed_isam1984@yahoo.com]
- *    Copyright 2013, 2014 (c)
+ *    Copyright 2013, 2014, 2015 (c)
  * 
  *    file: menu.c
  *    This file is part of Prime.
@@ -31,14 +31,8 @@
 
 #define EOL	10
 
+extern char *tmp, *tmp2;
 extern time_t tm;	//used to get system time to log it on shutdown
-extern int getKey();
-
-extern void cutMarked();
-extern void cutOne();
-extern void markAll();
-extern void unMarkAll();
-extern void setScreenColors(int FG, int BG);
 
 /********************************************
  * This procedure shows the FILE menu under
@@ -55,6 +49,8 @@ void showFileMenu(int visible)
   int endme = 0;
   if(visible == YES) 
   {	//show the file menu
+    //turn the cursor off
+    printf("\e[?25l");
     //activeWindow = FILE_MENU;
     drawBox(3, 2, 9, 19, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
@@ -77,9 +73,7 @@ void showFileMenu(int visible)
 	case(ESC_KEY):
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  endme = 1;
 	  break;
 	case('p'):
@@ -121,9 +115,7 @@ do_down:
 	 if(ALT) 
 	 {
 	  activeWindow = DIR_WIN;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  return;
 	 }
 	 else if(CTRL)
@@ -135,14 +127,11 @@ do_down:
 	case(RIGHT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening edit menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  //fprintf(log_file, "Opening edit menu..\n");
+	  refreshWindows();
 	  showEditMenu(YES);
 	  endme = 1;
 	  break;
-	//case(ALT_H_KEY):	//navigate to Help menu
 	case('h'):
 	  if(!ALT) break;
 	  goto do_left;
@@ -153,10 +142,8 @@ do_right:
 	case(LEFT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening help menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  //fprintf(log_file, "Opening help menu..\n");
+	  refreshWindows();
 	  showHelpMenu(YES);
 	  endme = 1;
 	  break;
@@ -164,6 +151,8 @@ do_left:
 	  if(fSelect == 0) 
 	  {	//user selected 'New directory'
 	    char *res = (char *) malloc(MAX_DIR_NAME_LEN);
+	    if(!res) { msgBox("Insufficient memory", OK, ERROR); endme = 1; break; }
+	    
 	    inputBox("Enter directory name to create:", "New Directory");
 	    strcpy(res, input);
 	    if(res != NULL) 
@@ -171,7 +160,7 @@ do_left:
 	      struct stat st;
 	      if(stat(res, &st) == -1) 
 	      {
-		mkdir(res, 0700);//check if it doesn't exist, create it
+		mkdir(res, 0775);//check if it doesn't exist, create it
 		fprintf(log_file, "Creating directory %s..\n", res);
 	      } 
 	      else 
@@ -180,8 +169,7 @@ do_left:
 		fprintf(log_file, "Directory %s already exists..\n", res);
 	      }
 	      scanDir(cwd);
-	      refreshFileView();
-	      refreshDirView();
+	      refreshWindows();
 	    } free(res);
 	    endme = 1;
 	    break;
@@ -196,32 +184,10 @@ do_left:
 	  }
 	  if(fSelect == 1) 
 	  {	//user selected 'Open location'
-	    char *res = inputBox("Enter directory path to open:", 
-				 "Open Location");
+	    char *res = file_open_location();
 	    if(res != NULL) 
 	    {
-	      //************** check for '~'
-	      char *tmp = (char *) malloc(MAX_DIR_NAME_LEN);
-	      strcpy(tmp, res);
-	      if(strchr(tmp, '~')) 
-	      {
-		char *tmp2 = (char *) malloc(MAX_DIR_NAME_LEN);
-		strcpy(tmp2, res+((strchr(res, '~')+1)-res));
-		struct passwd *pass;//will be used to find the home dir
-		if((pass = getpwuid(geteuid())))
-		  ;//printf("\nError: couldn't open home directory.\n");
-		strcpy(tmp, pass->pw_dir);
-		strcat(tmp, "/");
-		strcat(tmp, tmp2);
-		strcat(tmp, "\0");
-		free(tmp2);
-	      }//***************
-	      scanDir(tmp);
-	      refreshFileView();
-	      refreshDirView();
 	      endme = 1;
-	      fprintf(log_file, "Opening '%s'..\n", tmp);
-	      free(tmp);
 	      break;
 	    }
 	  }
@@ -241,22 +207,10 @@ do_left:
 	    i = msgBox("Are you sure you want to exit?", YES | NO, CONFIRM);
 	    if(i == YES) 
 	    { 	//exit gracefully
-	      fprintf(log_file, "Restoring terminal [ OK ]..\n");
-	      restoreTerminal(); 
-	      fprintf(log_file, "Closing files and Exiting Prime [ OK ]..\n");
-	      setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	      clearScreen(); 
-	      setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	      time_t tm = time(NULL);
-	      fprintf(log_file, "System time and date: %s", ctime(&tm));
-	      fcloseall();
-	      //freePrinters();
-	      exit(0); 
+	      exit_gracefully();
 	    }
 	  }
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  endme = 1;
 	  break;
       }
@@ -265,11 +219,12 @@ do_left:
   else 
   {		//hide the file menu
     activeWindow = DIR_WIN;
-    refreshFileView();
-    refreshDirView();
+    refreshWindows();
     return;
   }
-  fprintf(log_file, "Returning to main program from File Menu\n");
+  //turn the cursor on
+  printf("\e[?25h");
+  //fprintf(log_file, "Returning to main program from File Menu\n");
 }
 
 /********************************************
@@ -286,9 +241,11 @@ void showEditMenu(int visible)
   int i, ch, eSelect = 0;
   int endme = 0;
   if(visible == YES) 
-  {	//show the file menu
+  {	//show the edit menu
+    //turn the cursor off
+    printf("\e[?25l");
     //activeWindow = EDIT_MENU;
-    drawBox(3, 8, 11, 23, NULL, YES);
+    drawBox(3, 8, 11, 22, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
     fprintf(stdout, "\e[4;9H%s", editMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
@@ -310,9 +267,7 @@ void showEditMenu(int visible)
 	case(ESC_KEY):
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  endme = 1;
 	  break;
 	case('p'):
@@ -349,18 +304,14 @@ do_down:
 	  fprintf(stdout, "\e[%d;9H%s", eSelect+4, editMenu[eSelect]);
 	  fprintf(stdout, "\e[%d;22H", eSelect+4);
 	  break;
-	//case(ALT_E_KEY)://hide edit menu and return control to main program
 	case('e'):
 	 if(ALT) 
 	 {
 	  activeWindow = DIR_WIN;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  return;
 	 }
 	  break;
-	//case(ALT_F_KEY):	//navigate to File menu
 	case('b'):
 	  if(GNU_DOS_LEVEL < 2) break;
 	  if(!CTRL) break;
@@ -372,33 +323,26 @@ do_down:
 	case(LEFT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening file menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  //fprintf(log_file, "Opening file menu..\n");
+	  refreshWindows();
 	  showFileMenu(YES);
 	  endme = 1;
 	  break;
 	case('h'):
 	  if(!ALT) break;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening help menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  //fprintf(log_file, "Opening help menu..\n");
+	  refreshWindows();
 	  showHelpMenu(YES);
 	  endme = 1;
 	  break;
-	//case(ALT_H_KEY):	//navigate to Options menu
 	case('o'):
 	 if(!ALT) break;
 	 goto do_right;
 	case(RIGHT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening options menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  //fprintf(log_file, "Opening options menu..\n");
+	  refreshWindows();
 	  showOptionsMenu(YES);
 	  endme = 1;
 	  break;
@@ -411,19 +355,16 @@ do_right:
 	  if(eSelect == 5) clearSelection();
 	  if(eSelect == 6) 
 	  { 
-	    findFile(); 
-	    scanDir(cwd); 
+	    findFile();
+	    hideCursor();
+	    scanDir(cwd);
 	  }
 	  if(eSelect == 7) 
 	  {
 	    showPropertiesDialog();
 	    scanDir(cwd); 
 	  }
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //activeWindow = DIR_WIN;
-	  refreshFileView();
-	  refreshDirView();
-	  refreshBottomView();
+	  refreshAll();
 	  endme = 1;
 	  break;
       }
@@ -432,11 +373,12 @@ do_right:
   else 
   {		//hide the edit menu
     activeWindow = DIR_WIN;
-    refreshFileView();
-    refreshDirView();
+    refreshWindows();
     return;
   }
-  fprintf(log_file, "Returning to main program from Edit Menu\n");
+  //turn the cursor on
+  printf("\e[?25h");
+  //fprintf(log_file, "Returning to main program from Edit Menu\n");
 }
 
 
@@ -454,9 +396,11 @@ void showOptionsMenu(int visible)
   int i, ch, oSelect = 0;
   int endme = 0;
   if(visible == YES) 
-  {	//show the file menu
+  {	//show the options menu
+    //turn the cursor off
+    printf("\e[?25l");
     //activeWindow = HELP_MENU;
-    drawBox(3, 14, 6, 31, NULL, YES);
+    drawBox(3, 14, 7, 30, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
     fprintf(stdout, "\e[4;15H%s", optionsMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
@@ -475,10 +419,7 @@ void showOptionsMenu(int visible)
 	case(ESC_KEY):
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  endme = 1;
 	  break;
 	case('p'):
@@ -489,12 +430,10 @@ do_esc:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_up:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+4, optionsMenu[oSelect]);	//clear last selection
 	  oSelect --;
 	  if(oSelect < 0) oSelect = oTotal-1;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+4, optionsMenu[oSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;30H", oSelect+4);
 	  fflush(stdout);
@@ -507,22 +446,17 @@ do_up:
 	  if(GNU_DOS_LEVEL > 1) break;
 do_down:
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+4, optionsMenu[oSelect]);	//clear last selection
 	  oSelect++;
 	  if(oSelect >= oTotal) oSelect = 0;
 	  setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
-	  //setScreenColors(BLUE, BGWHITE);
 	  fprintf(stdout, "\e[%d;15H%s", oSelect+4, optionsMenu[oSelect]);	//highlight new selection
 	  fprintf(stdout, "\e[%d;30H", oSelect+4);
 	  fflush(stdout);
 	  break;
 	case('o'):	//hide the file menu and return control to main program
 	  if(!ALT) break;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  return;
 	  break;
 	case('h'):	//navigate to Help menu
@@ -531,20 +465,14 @@ do_down:
 	case(RIGHT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  showHelpMenu(YES);
 	  endme = 1;
 	  break;
 	case('f'):	//navigate to File menu
 	  if(CTRL && GNU_DOS_LEVEL > 1) goto do_right;
 	  if(!ALT) break;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  showFileMenu(YES);
 	  endme = 1;
 	  break;
@@ -558,39 +486,40 @@ do_right:
 	case(LEFT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  showEditMenu(YES);
 	  endme = 1;
 	  break;
 	case(ENTER_KEY):
 	  if(oSelect == 0) 
+	  {			//selected 'Properties'
+	    showPropertiesDialog();
+	    setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
+	    scanDir(cwd);
+	  }
+	  else if(oSelect == 1) 
 	  {			//selected 'Change colors'
 	    optionsMenu_Change_Colors();
 	  }
-	  else if(oSelect == 1) 
+	  else if(oSelect == 2) 
 	  {			//selected 'Reset config'
 	    optionsMenu_Reset_Config();
 	  }
 	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  //setScreenColors(WHITE, BGBLUE);
-	  refreshFileView();
-	  refreshDirView();
-	  //refreshBottomView();
+	  refreshWindows();
 	  endme = 1;
 	  break;
-	  //return;
       }
     }
   } 
   else 
   {		//hide the options menu
-    refreshFileView();
-    refreshDirView();
+    refreshWindows();
     return;
   }
+  //turn the cursor on
+  printf("\e[?25h");
+  //fprintf(log_file, "Returning to main program from Options Menu\n");
 }
 
 
@@ -608,9 +537,11 @@ void showHelpMenu(int visible)
   int i, ch, hSelect = 0;
   int endme = 0;
   if(visible == YES) 
-  {	//show the file menu
+  {	//show the help menu
+    //turn the cursor off
+    printf("\e[?25l");
     //activeWindow = HELP_MENU;
-    drawBox(3, 23, 8, 40, NULL, YES);
+    drawBox(3, 23, 8, 39, NULL, YES);
     setScreenColors(FG_COLOR[COLOR_HIGHLIGHT_TEXT], BG_COLOR[COLOR_HIGHLIGHT_TEXT]);
     fprintf(stdout, "\e[4;24H%s", helpMenu[0]);
     setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
@@ -629,9 +560,7 @@ void showHelpMenu(int visible)
 	case(ESC_KEY):
 	  if(GNU_DOS_LEVEL > 2) break;
 do_esc:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  endme = 1;
 	  break;
 	case('p'):
@@ -668,18 +597,14 @@ do_down:
 	  fprintf(stdout, "\e[%d;24H%s", hSelect+4, helpMenu[hSelect]);
 	  fprintf(stdout, "\e[%d;39H", hSelect+4);
 	  break;
-	//case(ALT_H_KEY)://hide help menu and return control to main program
 	case('h'):
 	 if(ALT) 
 	 {
 	  activeWindow = DIR_WIN;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  return;
 	 }
 	 break;
-	//case(ALT_F_KEY):	//navigate to File menu
 	case('f'):
 	 if(CTRL && GNU_DOS_LEVEL > 1) goto do_right;
 	 if(!ALT) break;
@@ -687,23 +612,16 @@ do_down:
 	case(RIGHT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_right:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening file menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  showFileMenu(YES);
 	  endme = 1;
 	  break;
 	case('e'):
 	  if(!ALT) break;
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening edit menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  showEditMenu(YES);
 	  endme = 1;
 	  break;
-	//case(ALT_E_KEY):	//navigate to Options menu
 	case('o'):
 	 if(!ALT) break;
 	 goto do_left;
@@ -714,17 +632,14 @@ do_right:
 	case(LEFT_KEY):
 	  if(GNU_DOS_LEVEL > 1) break;
 do_left:
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  fprintf(log_file, "Opening options menu..\n");
-	  refreshFileView();
-	  refreshDirView();
+	  refreshWindows();
 	  showOptionsMenu(YES);
 	  endme = 1;
 	  break;
 	case(ENTER_KEY):
 	  if(hSelect == 0) 
 	  {			//show README
-	    fprintf(log_file, "Opening README..\n");
+	    //fprintf(log_file, "Opening README..\n");
 	    showReadMe();
 	  } 
 	  else if(hSelect == 1) 
@@ -733,10 +648,9 @@ do_left:
 	  } 
 	  else if(hSelect == 3) 
 	  {		//show About msgbox
-	    fprintf(log_file, "Opening About dialog box..\n");
 	    char *x = " Prime for GNU/Linux. \n Developed by "
-		      "Mohammed Isam, \n 2013, 2014. ";
-	    msgBox(x, OK, INFO);
+		      "Mohammed Isam, \n 2013, 2014, 2015.";
+	    show_about(x);
 	  } 
 	  else if(hSelect == 2) 
 	  {	//show Quick reference
@@ -758,12 +672,7 @@ do_left:
 	    fprintf(stdout, "\e[17;7HPress any key to continue..");
 	    do {;} while(!getKey());
 	  }
-	  setScreenColors(FG_COLOR[COLOR_WINDOW], BG_COLOR[COLOR_WINDOW]);
-	  drawBox(1,1, SCREEN_H, SCREEN_W," Prime for GNU/Linux ",NO);
-	  drawMenuBar(2, 2, SCREEN_W-2);
-	  refreshFileView();
-	  refreshDirView();
-	  refreshBottomView();
+	  refreshAll();
 	  endme = 1;
 	  break;
       }
@@ -772,11 +681,12 @@ do_left:
   else 
   {		//hide the help menu
     activeWindow = DIR_WIN;
-    refreshFileView();
-    refreshDirView();
+    refreshWindows();
     return;
   }
-  fprintf(log_file, "Returning to main program from Help Menu\n");
+  //turn the cursor on
+  printf("\e[?25h");
+  //fprintf(log_file, "Returning to main program from Help Menu\n");
 }
 
 
@@ -785,155 +695,12 @@ do_left:
 ////////////////////////////////////////////////////////
 void showKeybindings() 
 {
-  int x = 2;
-  int y = 2;
-  int w = SCREEN_W-1;
-  int h = SCREEN_H-1;
-  FILE *KEYBINDINGS;
-  char buf[4096*4];	//buffer to hold data
-  int buf_len = 0;
-  
-  if(!(KEYBINDINGS = fopen("/usr/share/doc/gnudos/prime/keybindings", "r"))) 
+  int res = show_readme("/usr/share/doc/gnudos/prime/keybindings", " KEYBINDINGS ", GNU_DOS_LEVEL);
+  if(res != 0)
   {
-   if(!(KEYBINDINGS = fopen("/usr/local/share/doc/gnudos/prime/keybindings", "r"))) 
-   {
-    msgBox("Failed to open KEYBINDINGS file!.", OK, ERROR);
-    fprintf(log_file, "Failed to open /usr/share/doc/gnudos/prime/keybindings file..\n");
-    scanDir(cwd);
-    return;
-   }
+    res = show_readme("/usr/local/share/doc/gnudos/prime/keybindings", " KEYBINDINGS ", GNU_DOS_LEVEL);
   }
-  
-  if(!(buf_len = fread(buf, sizeof(char), sizeof(buf), KEYBINDINGS))) 
-  {
-    msgBox("Failed to read the KEYBINDINGS file!.", OK, ERROR);
-    fprintf(log_file, "Failed to open /usr/share/doc/gnudos/prime/keybindings file..\n");
-    scanDir(cwd);
-    return;
-  }
-  
-  drawBox(x, y, h, w, " KEYBINDINGS ", YES);
-  
-  y++; x++;
-  int i = 0, j = x;
-  int l = y;
-  int lineStart[60];
-  int firstVisLine = 0;
-  int cnt = 0;
-  //char *ch = (char *)malloc(5);
-  int ch;
-  char moreLines = 1;	//used as boolean to indicate if still more lines
-  lineStart[0] = 0;
-  
-  fprintf(stdout, "\e[%d;%dH", j, l);
-  //for(i = 0; i < buf_len; i++) {
-  while(j < (h-x+2)) 
-  {
-    if(buf[i] == EOL) 
-    { 
-      lineStart[++cnt] = i+1;
-      l=y;
-      fprintf(stdout, "\e[%d;%dH", ++j, l); 
-    } 
-    else 
-    { 
-      if(l > (w-y+2)) 
-      {
-	lineStart[++cnt] = i;
-	l=y;
-	fprintf(stdout, "\e[%d;%dH", ++j, l);
-      } putchar(buf[i]); l++;
-    }
-    if(++i >= buf_len) break;
-  }
-  
-  fflush(stdout);
-  //if the output chars are less than the buffer length,
-  //it means there are more lines to the output.
-  if(i <= buf_len) 
-  {
-    goto readMore;
-  } 
-  else 
-  {
-    do {;} while(!getKey());
-    goto end;
-    return;
-  }
-  
-readMore:
-  while(1) 
-  {
-    ch = getKey();
-    switch(ch) 
-    {
-      case('g'):
-	if(GNU_DOS_LEVEL < 3) break;
-	if(!CTRL) break;
-	goto do_esc;
-      case(ESC_KEY):
-	if(GNU_DOS_LEVEL > 2) break;
-do_esc:
-      case(ENTER_KEY):
-      case(SPACE_KEY):
-	goto end;
-	break;
-      case('p'):
-	if(GNU_DOS_LEVEL < 2) break;
-	if(!CTRL) break;
-	goto do_up;
-      case(UP_KEY):
-	if(GNU_DOS_LEVEL > 1) break;
-do_up:
-	if(firstVisLine == 0) break;	//can't go up--first line already
-	firstVisLine--;
-	break;
-      case('n'):
-	if(GNU_DOS_LEVEL < 2) break;
-	if(!CTRL) break;
-	goto do_down;
-      case(DOWN_KEY):
-	if(GNU_DOS_LEVEL > 1) break;
-do_down:
-	if(!moreLines) break;	//reached last line
-	firstVisLine++;
-	break;
-    }	//end of switch
-    //redraw the box with its contents
-    drawBox(x-1, y-1, h, w, " KEYBINDINGS ", YES);
-    j = x; l = y; cnt = firstVisLine; i = lineStart[firstVisLine];
-    
-    fprintf(stdout, "\e[%d;%dH", j, l);
-    while(j < (h-x+2)) 
-    {
-     if(buf[i] == EOL) 
-     { 
-      lineStart[++cnt] = i+1;
-      l=y;
-      fprintf(stdout, "\e[%d;%dH", ++j, l); 
-     } 
-     else 
-     { 
-      if(l > (w-y+2)) 
-      {
-	lineStart[++cnt] = i;
-	l=y;
-	fprintf(stdout, "\e[%d;%dH", ++j, l);
-      } putchar(buf[i]); l++;
-     }
-     if(++i >= buf_len) break;
-    }	//end of inner while
-    if(i <= buf_len) { moreLines = 1; }
-    else { moreLines = 0; }
-    fflush(stdout);
-
-  }	//end of outer while
-end:
-  fclose(KEYBINDINGS);
-  //free(buf);
   drawMenuBar(2, 2, SCREEN_W-2);	//draw main menu bar
-  scanDir(cwd);
-  //fflush(stdout);
   return;
 }
 
@@ -944,154 +711,36 @@ end:
  * ****************************************/
 void showReadMe() 
 {
-  int x = 2;
-  int y = 2;
-  int w = SCREEN_W-1;
-  int h = SCREEN_H-1;
-  FILE *README;
-  char buf[4096*4];	//buffer to hold data
-  int buf_len = 0;
-  
-  if(!(README = fopen("/usr/share/doc/gnudos/prime/README", "r"))) 
+  int res = show_readme("/usr/share/doc/gnudos/prime/README", " README ", GNU_DOS_LEVEL);
+  if(res != 0)
   {
-   if(!(README = fopen("/usr/local/share/doc/gnudos/prime/README", "r"))) 
-   {
-    msgBox("Failed to open README file!.", OK, ERROR);
-    fprintf(log_file, "Failed to open /usr/share/doc/gnudos/prime/README file..\n");
-    scanDir(cwd);
-    return;
-   }
+    res = show_readme("/usr/local/share/doc/gnudos/prime/README", " README ", GNU_DOS_LEVEL);
   }
-  
-  if(!(buf_len = fread(buf, sizeof(char), sizeof(buf), README))) 
-  {
-    msgBox("Failed to read the README file!.", OK, ERROR);
-    fprintf(log_file, "Failed to read /usr/share/doc/Prime/README file..\n");
-    scanDir(cwd);
-    return;
-  }
-  
-  drawBox(x, y, h, w, " README ", YES);
-  
-  y++; x++;
-  int i = 0, j = x;
-  int l = y;
-  int lineStart[60];
-  int firstVisLine = 0;
-  int cnt = 0;
-  int ch;
-  char moreLines = 1;	//used as boolean to indicate if still more lines
-  lineStart[0] = 0;
-  
-  fprintf(stdout, "\e[%d;%dH", j, l);
-  //for(i = 0; i < buf_len; i++) {
-  while(j <= (h-x+2)) 
-  {
-    if(buf[i] == EOL) 
-    { 
-      lineStart[++cnt] = i+1;
-      l=y;
-      fprintf(stdout, "\e[%d;%dH", ++j, l); 
-    } 
-    else 
-    { 
-      if(l >= (w-y+2)) 
-      {
-	lineStart[++cnt] = i;
-	l=y;
-	fprintf(stdout, "\e[%d;%dH", ++j, l);
-      } 
-      putchar(buf[i]); l++;
-    }
-    if(++i >= buf_len) break;
-  }
-  drawBox(x-1, y-1, h, w, " README ", NO);
-  
-  //if the output chars are less than the buffer length,
-  //it means there are more lines to the output.
-  if(i <= buf_len) 
-  {
-    goto readMore;
-  } 
-  else 
-  {
-    do {;} while(!getKey());
-    goto end;
-    return;
-  }
-  
-readMore:
-  while(1) 
-  {
-    ch = getKey();
-    switch(ch) 
-    {
-      case('g'):
-	if(GNU_DOS_LEVEL < 3) break;
-	if(!CTRL) break;
-	goto do_esc;
-      case(ESC_KEY):
-	if(GNU_DOS_LEVEL > 2) break;
-do_esc:
-      case(ENTER_KEY):
-      case(SPACE_KEY):
-	goto end;
-	break;
-      case('p'):
-	if(GNU_DOS_LEVEL < 2) break;
-	if(!CTRL) break;
-	goto do_up;
-      case(UP_KEY):
-	if(GNU_DOS_LEVEL > 1) break;
-do_up:
-	if(firstVisLine == 0) break;	//can't go up--first line already
-	firstVisLine--;
-	break;
-      case('n'):
-	if(GNU_DOS_LEVEL < 2) break;
-	if(!CTRL) break;
-	goto do_down;
-      case(DOWN_KEY):
-	if(GNU_DOS_LEVEL > 1) break;
-do_down:
-	if(!moreLines) break;	//reached last line
-	firstVisLine++;
-	break;
-    }	//end of switch
-    //redraw the box with its contents
-    drawBox(x-1, y-1, h, w, " README ", YES);
-    j = x; l = y; cnt = firstVisLine; i = lineStart[firstVisLine];
-    
-    fprintf(stdout, "\e[%d;%dH", j, l);
-    while(j <= (h-x+2)) 
-    {
-     if(buf[i] == EOL) 
-     { 
-      lineStart[++cnt] = i+1;
-      l=y;
-      fprintf(stdout, "\e[%d;%dH", ++j, l); 
-     } 
-     else 
-     { 
-      if(l >= (w-y+2)) 
-      {
-	lineStart[++cnt] = i;
-	l=y;
-	fprintf(stdout, "\e[%d;%dH", ++j, l);
-      } 
-      putchar(buf[i]); l++;
-     }
-     if(++i >= buf_len) break;
-    }	//end of inner while
-    if(i <= buf_len) { moreLines = 1; }
-    else { moreLines = 0; }
-    drawBox(x-1, y-1, h, w, " README ", NO);
-
-  }	//end of outer while
-end:
-  fclose(README);
-  //free(buf);
   drawMenuBar(2, 2, SCREEN_W-2);	//draw main menu bar
-  scanDir(cwd);
   return;
 }
+
+char *file_open_location()
+{
+  char *res = inputBox("Enter directory path to open:", "Open Location");
+  if(res != NULL)
+  {
+    //************** check for '~'
+    strcpy(tmp, res);
+    if(strchr(tmp, '~'))
+    {
+      strcpy(tmp2, res+((strchr(res, '~')+1)-res));
+      struct passwd *pass;//will be used to find the home dir
+      if((pass = getpwuid(geteuid())))
+	return NULL;
+      strcpy(tmp, pass->pw_dir);
+      strcat(tmp, "/");
+      strcat(tmp, tmp2);
+      strcat(tmp, "\0");
+    }//***************
+    scanDir(tmp);
+    refreshWindows();
+    return tmp;
+  }
+  return NULL;
+}
\ No newline at end of file
